///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS FOR DATA LOADING AND SYNCRHONIZATION
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    // Declare global counters to keep track of all db.transaction calls so that we
    // can detect when the action fails to execute on iOS because of a bug in iOS where
    // user scrolling or tapping the screen causes the database access to fail.  In this
    // failure mechanism, the "success" callback executes but the transaction action 
    // does not.  Therefore we use these counters to detect if the transaction fired.  If
    // it does not, then we can use the success callback to retry.
    DB_IOS_RETRY_LIMIT = 3;
    var global_db_ios_fail_DataLoadFsm_1;
    var global_db_ios_fail_DataLoadFsm_6;
    var global_db_ios_fail_NavBtSet_1;
    var global_db_ios_fail_AddPreg_1;
    var global_db_ios_fail_AddPreg_1;
    var global_db_ios_fail_UserPickPreg_1;
    var global_db_ios_fail_UserPickGroup_1;
    var global_db_ios_fail_FindNextPrev_1;
    var global_db_ios_fail_NavBtAnalysisScreens_1;
    var global_db_ios_fail_NavBtAnalysisScreens_6;
    var global_db_ios_fail_loadCalSubscreens_1;
    var global_db_ios_fail_navBtCommScreens_1;
    var global_db_ios_fail_SyncNow_1;
    var global_db_ios_OpenDbAndExec_1;
    var global_db_ios_checkPw_1;
    var global_db_ios_fail_userPickCyc_2;
    var global_db_ios_fail_replotGraphCht;
    var global_db_ios_fail_updateLastCalScreen_1;
    var global_db_ios_fail_UploadSyncToServ_3;
    var global_db_ios_fail_UploadSyncToServ_8;
    var global_db_ios_fail_UploadSyncToServ_13;
    var global_db_ios_fail_UploadSyncToServ_18;
    var global_db_ios_fail_InitStartup_1;
    var global_db_ios_fail_CheckIfDis_1;
    var global_db_ios_fail_LoadTipOfDay_1;
    var global_db_ios_fail_TipOfDayOk_1;

    var global_suspend_scrolling = false;
    var global_suspend_scrolling_counter = 1;

    // Function enables or disables scrolling
    function suspend_scrolling(enable_or_disable_not, scroll_counter_snapshot)
    {
        // Only necessary if this is iOS because the bug only exists in iOS database accesses
        if (global_is_ios)
        {
            // Disable Scrolling
            if (enable_or_disable_not == true)
            {
                // Increment call counter
                global_suspend_scrolling_counter = global_suspend_scrolling_counter + 1;
                
                // Disable scrolling
                global_suspend_scrolling = true;
                printDebug('scrolling OFF (req #' + global_suspend_scrolling_counter + ')');
                
                // Set timeout to re-enable if all else fails and we get locked up/stuck
                setTimeout("suspend_scrolling(false, " + global_suspend_scrolling_counter + ");", 1000);
                
                // Return the value for use with parallelism calls.  They'll want to know the value so they can pass it back when trying
                // to turn it off (to let this function know if someone else later made a similar request).
                return global_suspend_scrolling_counter;
            }
            else   // Re-enable scrolling
            {
                if (scroll_counter_snapshot == 0)
                {
                    // Re-enable scrolling (because of an explicit command to do so).  This is used when sequential operations are occurring
                    // and it is clear when you want to turn it off and back on
                    global_suspend_scrolling = false;
                    printDebug('scrolling back ON (req #' + global_suspend_scrolling_counter + ')');
                }
                else if(global_suspend_scrolling_counter <= scroll_counter_snapshot)
                {
                    //Re-enable scrolling (because the timeout callback expired)
                    global_suspend_scrolling = false;
                    printDebug('scrolling back ON - timeout or parallelism case reached (req #' + global_suspend_scrolling_counter + ')');
                }
                else
                {
                    // A later call was made to resuspend scrolling after this timeout was set, so ignore this earlier timeout request (and wait for the second one that will come later)
                    printDebug('scrolling timeout, but not enabled bc of later call (req #' + scroll_counter_snapshot + ' superceded by req #' + global_suspend_scrolling_counter + ')');
                }
                return;
            }
        }
    }

    // To call to load a Calendar, set:
    //          targetCalendar_OR_Analysis_NOT = true
    //          start_date_as_int = global_calendar_start_int
    //          end_date_as_int = global_calendar_last_int
    // To call to load an Analysis cycle, set: 
    //          targetCalendar_OR_Analysis_NOT = false
    //          start_date_as_int = global_selected_cycle_as_number
    //          end_date_as_int = global_selected_cycle_end_as_number
    function data_load_fsm(state, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, param_1, param_2, param_3)
    {
        var tx = param_1;
        var results = param_2;
        var strSql = ''; 
        var number_of_days = end_date_as_int - start_date_as_int + 1; 
        var LCS_PART_1_STATE_8 = 8;
        var LCS_PART_2_STATE_9 = 9;
        var LCS_PART_3_STATE_10 = 10;
        var LCS_PART_4_STATE_11 = 11;
        var LCS_PART_5_STATE_12 = 12;
        var LCS_PART_6_STATE_13 = 13;
        var LCS_AFTER_PARALLEL_EXEC_STATE_14 = 14;
        
        // Declare constants
        const NO_CHANGE=0;
        const CHANGED=1;
        const PENDING_UPDATE=2;    

        // Before we move on, make sure that the user hasn't navigated to a different cycle/month such that the info we are
        // loading is now irrelevant anyway
        if ((targetCalendar_OR_Analysis_NOT == true) && (start_date_as_int != global_calendar_start_int))
        {
            // The calendar date changed, so just abandon and move on
            printDebug('Abandoning calendar load of ' + start_date_as_int + ' because user navigated to ' + global_calendar_start_int);
            return;
        }
        if ((targetCalendar_OR_Analysis_NOT == false) && (start_date_as_int != global_selected_cycle_as_number))
        {
        printDebug('targetCalendar_OR_Analysis_NOT=' + targetCalendar_OR_Analysis_NOT);
            // The select cycle changed, so just abandon irrelevant one we were querying previously and move on
            printDebug('Abandoning cycle load of ' + start_date_as_int + ' because user navigated to ' + global_selected_cycle_as_number);
            return;
        }   
        if ((state > 1) && (caller_url != GetCurrentPage()))
        {
            // The page was changed during processing, so just abandon the query (else we may get errors about DIVs and things we can't find)
            printDebug('Abandoning data_load_fsm because page changed during processing');
            return;
        }
        
        // Callback State Machine
        var currTime = new Date();
        printDebug("State #" + state + " of data_load_fsm " + appendTimeStamp(currTime));
        switch(state)
        {
        ////////////////////////////////
        // STEP 1 - Initialize Screen //
        ////////////////////////////////
        case 1:
LogCall('a1');
            var tempDate = new Date();
            g_Performance_Debug_Start_Time = tempDate.getTime();
            printStatus('Initializing');
            // Clear status text
            clearInfoMessage();
           
            if (targetCalendar_OR_Analysis_NOT)
            {
                // Initialize globals
                for (var i=0; i<number_of_days; i++)
                {
                    mens_days[i] = 0;
                    recorded_data[i] = 0;
                }
                // Clear the entire calendar
                for (var i=1; i<=number_of_days; i++)
                {
                    // Blank everything on the calendar out
                    if (BUILD_TARGET == BUILD_4_CCL)
                    {
                        document.getElementById('CalendarDate_' + i).style.background = '';   // to clear ccl gradients
                        document.getElementById('CalendarDateDay_' + i).style.fontStyle='normal';  // to clear ccl start of cycles
                        document.getElementById('CalendarDateDay_' + i).style.textShadow='none';
                        document.getElementById('CalendarDateDay_' + i).style.fontSize='20px';
                    }
                    document.getElementById('CalendarDate_' + i).style.backgroundColor = '#FFFFFF';
                    document.getElementById('CalendarDateTest_' + i).innerHTML = '';
                    document.getElementById('CalendarDateData_' + i).innerHTML = '';
                }
            }
            
            // Open database
            db = window.openDatabase("Database", "1.0", "MyFertilityCharts Database", 2500000);
            global_db_ios_fail_DataLoadFsm_1 = DB_IOS_RETRY_LIMIT;
            db.transaction(     function(tx) 
                                {
                                    global_db_ios_fail_DataLoadFsm_1 = -1;
                                    data_load_fsm(state+1, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, GetCurrentPage(), tx, '', '');
                                },
                                function(tx, err)
                                {
                                    crash_and_burn(err, 'Could not open local database.  Please try again.  If this problem persists, you may want to try going to the settings page and re-associating with your account.');
                                },
                                function(tx)
                                {
                                    // Success callback refires if the iOS bug failed to fire the transaction code because the user scrolled or tapped.
                                    if (global_db_ios_fail_DataLoadFsm_1 > 0)
                                    {
                                        global_db_ios_fail_DataLoadFsm_1 = global_db_ios_fail_DataLoadFsm_1 - 1;
                                        printDebug('iOS db.transaction failed data_load_fsm state 1. Retry #' + (DB_IOS_RETRY_LIMIT - global_db_ios_fail_DataLoadFsm_1));
                                        data_load_fsm(state, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, '', '', '', '');
                                    }
                                    else if (global_db_ios_fail_DataLoadFsm_1 == 0)
                                    {
                                        printDebug('<font color=Red>Giving up after hitting max retries for iOS db bug.</font>');
                                    }
                                    else
                                    {
                                        successCB();
                                    }
                                }
                                
                           );
            break;
            
        case 2:
LogCall('a2');
            // Check user settings in the User_Info_DB
            strSql = 'SELECT * FROM User_Info_DB;';
            suspend_scrolling(true, '');
            tx.executeSql(  strSql, 
                            [], 
                            function(tx, results) 
                            {
                                suspend_scrolling(false, 0);
                                data_load_fsm(state+1, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, results, '');
                            },
                            function(tx, err)
                            {
                                crash_and_burn(err, 'Error querying local UserInfo database.  Please try again.  If this problem persists, you may want to try going to the settings page and re-associating with your account.');
                            }
                         );
            break;
        
        case 3:
LogCall('a3');
            // Retrieve results of query
            var len = results.rows.length;
            if (len == 0)
            {
                printDebug("Error - Missing user settings table!");
                global_EnableRemoteAnalysis = 0;
                global_SyncInterval = 3600;
                printInfoMessage('Error querying local UserInfo database.  Please try again.  If this problem persists, you may want to try going to the settings page and re-associating with your account.');
            }
            else
            {
                global_EnableRemoteAnalysis = results.rows.item(0).enable_remote_analysis;
                global_SyncInterval = results.rows.item(0)['sync_interval'];
                global_username = results.rows.item(0)['un'];
                global_password = results.rows.item(0)['pw'];
                global_t_units = results.rows.item(0)['t_units'];
                global_server_prefix = results.rows.item(0)['server_prefix'];
                global_NormalTempTime = results.rows.item(0)['temp_time'];
                global_CalShadeFert = results.rows.item(0)['cal_shade'];
                global_Debug_Enabled = results.rows.item(0)['debug'];
                global_CalOv = results.rows.item(0)['cal_ov'];
                global_svr_info_msg = results.rows.item(0)['svr_info_msg'];
                global_svr_info_msg_shown = results.rows.item(0)['svr_info_msg_shown'];
                global_survey_id = results.rows.item(0)['survey_id'];
                if (BUILD_TARGET == BUILD_4_CCL)
                {
                    global_active_subscription = results.rows.item(0)['sub_ctr'];
                    global_user_height = results.rows.item(0)['height'];
                }
            }

            // Call function to render user interfaced based on local database
            if (targetCalendar_OR_Analysis_NOT)
            {
                // This call is for the calendar level of screens.  Load content for those.
                RenderCalendarFromLocalDatabase(     1,
                                        function(tx) 
                                        {
                                            data_load_fsm(state+1, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, '', '', false);
                                        },
                                        tx,
                                        '',
                                        false
                                   );
           }
           else
           {
                // This call is for the Charting & Analysis level of screens.  Load content for those.
                RenderAnalysisScreensFromLocalDatabase(     1,
                                        function(tx) 
                                        {
                                            data_load_fsm(state+1, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, '', '', false);
                                        },
                                        tx,
                                        '',
                                        '',
                                        false,
                                        start_date_as_int + global_selected_cycle_offset, 
                                        end_date_as_int,
                                        '',
                                        ''
                                   );
           }
           break;
           
        case 4:
LogCall('a4');
            // Note time that local database finished being loaded if we are in Analysis mode
            if (targetCalendar_OR_Analysis_NOT == false)
            {
                var currTime = new Date(); 
                g_LoadTimestamps['same_cycle'] = currTime.getTime();
            }
            
            // Decide if we should re-load the remote server content because it has been a while (the timestamp will be
            // old) or because the user changed something (the timestamps will have been wiped out). 
            var continue_loading = false;
            if (targetCalendar_OR_Analysis_NOT)
            {
                continue_loading = does_timestamp_say_to_reload(calendar_loaded_date.getMonth() + '-' + calendar_loaded_date.getFullYear());
            }
            else
            {
                continue_loading = does_timestamp_say_to_reload(PrintDate(NumberToDate(start_date_as_int)));
            }
            
            if (continue_loading == false)
            {
                // No need to go any further.  We synchronized with the server fairly recently.
                printStatus('');
                CheckAndWarnIfOfflineOnly();
printDebug('NO NEED TO RELOAD');
            }
            else
            {
printDebug('Need to reload - Continuing');
                // See if remote analysis is enabled
                if (CheckAndWarnIfOfflineOnly() == false)
                {
                    // Start downloading data from the remote server
                    UploadSyncToServer(     1,
                                            function(tx) 
                                            {
                                                data_load_fsm(state+1, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, '', '');
                                            },
                                            tx,
                                            '',
                                            0
                                       );
                }          
            }
            HideAct();
            break;

        //////////////////////////////////////////////////
        // STEP 3 - Download Records from Remote Server //
        //////////////////////////////////////////////////
        case 5:
LogCall('a5');
            // Continue loading
            printStatus(WAITING_STATUS_MSG);
            var uploadParameters = {};  // Declare name value pair list to specify values to upload to server
            var url_str = global_server_prefix + 'apps_get_data.aspx?' + new Date().getTime();
            
            // Specify values to download - Calendar Highlighting and Numbering
            uploadParameters['un'] = global_username;
            uploadParameters['pw'] = global_password;
            uploadParameters['debug'] = global_Debug_Enabled;
            uploadParameters['app_ver'] = global_app_ver_str;
            uploadParameters['device_id'] = global_device_id_str;
            uploadParameters['fdata'] = '1';
            uploadParameters['notes'] = '1';
            uploadParameters['event'] = '1';
            uploadParameters['ev_names'] = '1';
            uploadParameters['t_units'] = global_t_units;
            uploadParameters['chart'] = '1';
            uploadParameters['fpct'] = '1';
            uploadParameters['calendar'] = '1';
            uploadParameters['preg'] = '1';
            uploadParameters['cstats'] = '1';
            if (targetCalendar_OR_Analysis_NOT)
            {
                // Calendar screen specific fields
                uploadParameters['date'] = PrintDate(NumberToDate(start_date_as_int));
                uploadParameters['end_date'] = PrintDate(NumberToDate(end_date_as_int));
            }
            else
            {
                // Charting & Analysis screen specific fields
                uploadParameters['date'] = PrintDate(NumberToDate(start_date_as_int));
                uploadParameters['end_date'] = PrintDate(NumberToDate(end_date_as_int));
                uploadParameters['ovdates'] = '1';
                uploadParameters['confidence'] = '1';
                uploadParameters['fwin'] = '1';
            }

            // Use jQuery to upload the data to the server
            $.ajax({
                type: 'POST',
                // The URL of the request
                url: url_str,
                // Values to upload
                data: uploadParameters,
                // The success callback with result from server
                success: function(newContent) {
                    // Handles response this function and advance to the next state.  Notice that the tx
                    // cannot be passed to this legitimately because it is not being created as a call
                    // to this registered function (it only has a newContent argument) because this 
                    // callback has nothing to do with databases.  Therefore we will have to re-establish
                    // the tx variable when we want to resume database operations.
                    data_load_fsm(state+1, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, '', '', newContent);
                },
                dataType: 'html'
            });
            break;
            
        case 6:
LogCall('a6');
            var error_or_warning;
            var continue_execution;
            var newContent = param_3;
            var continue_execution;
            
            printStatus('Analyzing');
            
            // Validate the server response before continuing (to check for fatal errors in the response)
            error_or_warning = CheckForStandardErrors(newContent);
            continue_execution = HandleServerErrorOrWarning(error_or_warning);
            if (continue_execution == 0)
            {
                // Error occurred that warrants giving up and stopping further execution
                printStatus('Server Offline?');
                hideStatusSpinner();
            }
            else
            {
                // Re-generate the tx variable so we can continue to perform database operations.  It was lost
                // in the prior state because the last callback into the state machine involved a remote server
                // transfer and not a local database access.  Therefore the local database parameter was not
                // carried forward and needs to be regenerated.
                global_db_ios_fail_DataLoadFsm_6 = DB_IOS_RETRY_LIMIT;
                db.transaction(     function(tx) 
                                    {
                                        global_db_ios_fail_DataLoadFsm_6 = -1;
                                        data_load_fsm(state+1, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, param_2, param_3);
                                    },
                                    function(tx, err)
                                    {
                                        crash_and_burn(err, 'Could not open local database.  Please try again.  If this problem persists, you may want to try going to the settings page and re-associating with your account.');
                                    },
                                    function(tx)
                                    {
                                        // Success callback refires if the iOS bug failed to fire the transaction code because the user scrolled or tapped.
                                        if (global_db_ios_fail_DataLoadFsm_6 > 0)
                                        {
                                            global_db_ios_fail_DataLoadFsm_6 = global_db_ios_fail_DataLoadFsm_6 - 1;
                                            printDebug('iOS db.transaction failed data_load_fsm state 6. Retry #' + (DB_IOS_RETRY_LIMIT - global_db_ios_fail_DataLoadFsm_6));
                                            data_load_fsm(state, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, param_1, param_2, param_3);
                                        }
                                        else if (global_db_ios_fail_DataLoadFsm_6 == 0)
                                        {
                                            printDebug('<font color=Red>Giving up after hitting max retries for iOS db bug.</font>');
                                        }
                                        else
                                        {
                                            successCB();
                                        }
                                    }
                               );
            }
            break;
        
        case 7:
LogCall('a7');
            var newContent = param_3;
            var start_date_as_int;
            var end_date_as_int;
            
            // Initialize to assume we have to do everything
            LoadCalScreen_InterpDB = 0;
            LoadCalScreen_DataRecsDB = 0;
            LoadCalScreen_EventSettingsDB = 0;  
            LoadCalScreen_CyclesDB = 0;
            LoadCalScreen_UserInfoDB = 0;
            LoadCalScreen_PregDB = 0;
            
            // Suspend all scrolling on iPhone until all database queries complete (we reach state 14).  We can safely do this because the database queries
            // should be very fast and we should get to state 14 quickly because no server interaction occurs.
            suspend_scrolling(true, '');
            
            ////////////////////////////////////////
            // First Parallel Path - Interp_DB //
            ////////////////////////////////////////
            LoadCalScreen_InterpDB = 0;

            // Before we parse the response, first query the local database so we know how to update it as we parse the new data
            strSql = 'SELECT * FROM Interp_DB WHERE record_date >= ' + start_date_as_int 
            strSql += ' AND record_date <= ' + end_date_as_int + ' ORDER BY record_date;';
            // Note: suspend_scrolling(true, '') already called above in a blanket call for States 7 thru 14
            tx.executeSql(  strSql, 
                            [], 
                            function(tx, results) 
                            {
                                data_load_fsm(LCS_PART_1_STATE_8, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, results, newContent);
                            },
                            function(tx, err)
                            {
                                crash_and_burn(err, 'Error re-querying local Interp database.  Please try again.  If this problem persists, you may want to try going to the settings page and re-associating with your account.');
                            }
                         ); 

            /////////////////////////////////////////
            // Second Parallel Path - Data_Recs_DB //
            /////////////////////////////////////////
            LoadCalScreen_DataRecsDB = 0;
            
            // Before we continue to parse the rest of the response, query the local database so we know how to update it as we parse the new data
            strSql = 'SELECT * FROM Data_Recs_DB WHERE record_date >= ' + start_date_as_int
            strSql += ' AND record_date <= ' + end_date_as_int + ' ORDER BY record_date;';
             // Note: suspend_scrolling(true, '') already called above in a blanket call for States 7 thru 14
            tx.executeSql(  strSql, 
                            [], 
                            function(tx, results) 
                            {
                                data_load_fsm(LCS_PART_2_STATE_9, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, results, newContent);
                            },
                            function(tx, err)
                            {
                                crash_and_burn(err, 'Error re-querying local DataRecs database.  Please try again.  If this problem persists, you may want to try going to the settings page and re-associating with your account.');
                            }
                         );

            /////////////////////////////////////////////
            // First Parallel Path - Event_Settings_DB //
            /////////////////////////////////////////////
            LoadCalScreen_EventSettingsDB = 0;  

            if (does_timestamp_say_to_reload('events'))
            {
                // Before we continue to parse the rest of the response, query the local database so we know how to update it as we parse the new data
                strSql = 'SELECT * FROM Event_Settings_DB';
                 // Note: suspend_scrolling(true, '') already called above in a blanket call for States 7 thru 14
                tx.executeSql(  strSql, 
                                [], 
                                function(tx, results) 
                                {
                                    data_load_fsm(LCS_PART_3_STATE_10, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, results, newContent);
                                },
                                function(tx, err)
                                {
                                    crash_and_burn(err, 'Error re-querying local EventSettings database.  Please try again.  If this problem persists, you may want to try going to the settings page and re-associating with your account.');
                                }
                             );
            }
            else
            {
                // We don't need to load events, so by definition we are done already
                LoadCalScreen_EventSettingsDB = 1;
printDebug('no need to reload events');
            }            
            
            /////////////////////////////////////////////
            // Fourth Parallel Path - Cycles_DB //
            /////////////////////////////////////////////
            if (targetCalendar_OR_Analysis_NOT)
            {
                LoadCalScreen_CyclesDB = 1;  
            }
            else
            {
                LoadCalScreen_CyclesDB = 0;
                // Before we continue to parse the rest of the response, query the local database so we know how to update it as we parse the new data
                strSql = 'SELECT * FROM Cycles_DB WHERE cyc_started=' + start_date_as_int + ';';
                 // Note: suspend_scrolling(true, '') already called above in a blanket call for States 7 thru 14
                tx.executeSql(  strSql, 
                                [], 
                                function(tx, results) 
                                {
                                    data_load_fsm(LCS_PART_4_STATE_11, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, results, newContent);
                                },
                                function(tx, err)
                                {
                                    crash_and_burn(err, 'Error re-querying local CyclesDB database.  Please try again.  If this problem persists, you may want to try going to the settings page and re-associating with your account.');
                                }
                             );
             }
                         
            /////////////////////////////////////////////
            // Fifth Parallel Path - User_Info_DB //
            /////////////////////////////////////////////
            if (targetCalendar_OR_Analysis_NOT)
            {
                LoadCalScreen_UserInfoDB = 1;  
            }
            else
            {
                LoadCalScreen_UserInfoDB = 0;
                // Before we continue to parse the rest of the response, query the local database so we know how to update it as we parse the new data
                strSql = 'SELECT * FROM User_Info_DB';
                 // Note: suspend_scrolling(true, '') already called above in a blanket call for States 7 thru 14
                tx.executeSql(  strSql, 
                                [], 
                                function(tx, results) 
                                {
                                    data_load_fsm(LCS_PART_5_STATE_12, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, results, newContent);
                                },
                                function(tx, err)
                                {
                                    crash_and_burn(err, 'Error re-querying local UserInfoDB database ' + state + '.  Please try again.  If this problem persists, you may want to try going to the settings page and re-associating with your account.');
                                }
                             );
             }
             
            /////////////////////////////////////////////
            // Sixth Parallel Path - Preg_DB //
            /////////////////////////////////////////////
            if (does_timestamp_say_to_reload('preg'))
            {
                LoadCalScreen_PregDB = 0;
                // Before we continue to parse the rest of the response, query the local database so we know how to update it as we parse the new data
                strSql = 'SELECT * FROM Preg_DB ORDER BY preg_started ASC';
                 // Note: suspend_scrolling(true, '') already called above in a blanket call for States 7 thru 15
                tx.executeSql(  strSql, 
                                [], 
                                function(tx, results) 
                                {
                                    data_load_fsm(LCS_PART_6_STATE_13, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, results, newContent);
                                },
                                function(tx, err)
                                {
                                    crash_and_burn(err, 'Error re-querying local UserInfoDB database ' + state + '.  Please try again.  If this problem persists, you may want to try going to the settings page and re-associating with your account.');
                                }
                             );
            }
            else
            {
                // We don't need to reload, so by definition we are done already
                LoadCalScreen_PregDB = 1; 
            }
            
            // Done kicking off all parallel paths.  Wait for them to execute
            break;                  
            
        case LCS_PART_1_STATE_8:
LogCall('a8');
            /////////////////////////////////////
            // PART 1 - Compare with Interp_DB //
            /////////////////////////////////////

            // Analyze server response and update local database with necessary changes (Interp_DB)
            var desiredStrStart=new Array();        // Start delimeter to look for in server response
            var desiredStrEnd=new Array();          // End delimeter to look for in server response
            var dbFieldObj=new Array();             // Pointers to objects describing database fields
            var respParamsStr=new Array();          // Placeholder for the parsed response found in the server returned content.
            
            var index2_Color_field=new Array();
            var index2_Cd_field=new Array();
            var index2_Dpo_field=new Array();
            var index2_PregCnt_field=new Array();
            var index2_RecTest_field=new Array();
            var index2_BBTOvCLen_field=new Array();
            var index2_MonOv_field=new Array();
            var index2_CMOv_field=new Array();
            var index2_AllOv_field=new Array();
            var index2_FWin_field=new Array();
            var index2_FPctAll_field=new Array();
            var index2_FPctCm_field=new Array();
            var index2_FPctSaliva_field=new Array();
            var index2_FPctMon_field=new Array();
            var index2_FPctBbt_field=new Array();
            var index2_FPctRhy_field=new Array();
            var index2_FPctLh_field=new Array();
            var index2_FPctCervix_field=new Array();
            var index2_BillInterp_field=new Array();
            var index2_CclStCov_field=new Array();
            
            var newContent = param_3;
            var local_db_counter = 0;
            var len = results.rows.length;
            var rec_date_int;

            // Prepare variables to extract from response - organized to happen in order server returns them for efficiency                    
            for (var i=1; i<=number_of_days; i++)
            {
				index2_FPctAll_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FPCT_ALL_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_FPctAll;
                
                index2_FPctCm_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FPCT_CM_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_FPctCm;
                
                index2_FPctSaliva_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FPCT_SALIVA_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_FPctSaliva;
                
                index2_FPctMon_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FPCT_MON_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_FPctMon;
                
                index2_FPctBbt_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FPCT_BBT_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_FPctBbt;
                
                index2_FPctRhy_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FPCT_RHY_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_FPctRhy;
                
				index2_FPctCervix_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FPCT_CERVIX_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_FPctCervix;
				
                index2_FPctLh_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FPCT_LH_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_FPctLh;
                
				index2_BillInterp_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='BILL_ITP_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_BillInterp;
			}
			for (var i=1; i<=number_of_days; i++)
            {
				index2_Color_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='COLOR_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_Color;
				
				index2_PregCnt_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='PREG_CNT_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_PregCnt;
			}
			for (var i=1; i<=number_of_days; i++)
            {
				index2_RecTest_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='REC_TEST_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_RecTest;
			}
			for (var i=number_of_days; i>=1; i--)
            {
				index2_MonOv_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='MON_OV_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_MonOv;
				
				index2_BBTOvCLen_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='BBT_COV_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_BBTOvCLen;
                
				index2_CclStCov_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='CCLST_COV_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_BBTCclCovLine;

				index2_CMOv_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='CM_OV_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_CMOv;
				
				index2_AllOv_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='ALL_OV_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_AllOv;
				
				index2_Cd_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='CD_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_Cd;
				
				index2_Dpo_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='DPO_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_Dpo;
				
				index2_FWin_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FWIN_' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbInterpDBField_FWin;
			}
			           
            // Parse the response to find the desired values
            respParamsStr = parse_server_response(newContent, desiredStrStart, desiredStrEnd);

            // Loop through the days spanning the range
            for (var i=1; i<=number_of_days; i++)
            {
                local_db_match = 0;
                var QueryDetails = {execute_strSql: 0, strSql: ''};
                
                // Convert missing fields to neutral values (likely missing due to being a non-subscriber)
                respParamsStr[index2_Color_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_Color_field[i]], '-');
                respParamsStr[index2_Cd_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_Cd_field[i]], -1);
                respParamsStr[index2_Dpo_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_Dpo_field[i]], -1);
                respParamsStr[index2_PregCnt_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_PregCnt_field[i]], -1);
                respParamsStr[index2_BBTOvCLen_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_BBTOvCLen_field[i]], 0);
                respParamsStr[index2_CclStCov_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_CclStCov_field[i]], 0);
                respParamsStr[index2_MonOv_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_MonOv_field[i]], 0);
                respParamsStr[index2_CMOv_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_CMOv_field[i]], 0);
                respParamsStr[index2_AllOv_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_AllOv_field[i]], 0);
                respParamsStr[index2_FWin_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_FWin_field[i]], 0);
                respParamsStr[index2_FPctAll_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_FPctAll_field[i]], 0);
                respParamsStr[index2_FPctCm_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_FPctCm_field[i]], 0);
                respParamsStr[index2_FPctSaliva_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_FPctSaliva_field[i]], 0);
                respParamsStr[index2_FPctMon_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_FPctMon_field[i]], 0);
                respParamsStr[index2_FPctBbt_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_FPctBbt_field[i]], 0);
                respParamsStr[index2_FPctRhy_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_FPctRhy_field[i]], 0);
                respParamsStr[index2_FPctLh_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_FPctLh_field[i]], 0);
                respParamsStr[index2_BillInterp_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_BillInterp_field[i]], '-');
                respParamsStr[index2_FPctCervix_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_FPctCervix_field[i]], 0);
                respParamsStr[index2_RecTest_field[i]] = IfEmptyAssignAlternate(respParamsStr[index2_RecTest_field[i]], 0);
                
                // See if the local database already has a value that matches this
                if (local_db_counter < len)
                {
                    rec_date_int = results.rows.item(local_db_counter)['record_date'];
                    if ((rec_date_int - start_date_as_int + 1) == i)
                    {
                        // This is a match.  We must update the existing database entry and consider "_chg" flag values
                        QueryDetails.strSql = 'UPDATE Interp_DB SET ';
                        
                        // Create a list of database elements we need to consider
                        var relevantIndices=new Array();
                        relevantIndices[0] = index2_Color_field[i];
                        relevantIndices[1] = index2_Cd_field[i];
                        relevantIndices[2] = index2_Dpo_field[i];
                        relevantIndices[3] = index2_PregCnt_field[i];
                        relevantIndices[4] = index2_BBTOvCLen_field[i];
                        relevantIndices[5] = index2_MonOv_field[i];
                        relevantIndices[6] = index2_CMOv_field[i];
                        relevantIndices[7] = index2_AllOv_field[i];
                        relevantIndices[8] = index2_FWin_field[i];
                        relevantIndices[9] = index2_FPctAll_field[i];
                        relevantIndices[10] = index2_FPctCm_field[i];
                        relevantIndices[11] = index2_FPctSaliva_field[i];
                        relevantIndices[12] = index2_FPctMon_field[i];
                        relevantIndices[13] = index2_FPctBbt_field[i];
                        relevantIndices[14] = index2_FPctRhy_field[i];
                        relevantIndices[15] = index2_FPctLh_field[i];
                        relevantIndices[16] = index2_FPctCervix_field[i];   
                        relevantIndices[17] = index2_RecTest_field[i];  
                        if (BUILD_TARGET == BUILD_4_CCL)
                        {
                            relevantIndices[18] = index2_CclStCov_field[i];
                            relevantIndices[19] = index2_BillInterp_field[i];                        
                        }
                        
                        // Call a function to walk through the relevantIndices and build the appropriate query string and indicate if it should execute using QueryDetails
                        QueryDetails = loopThruRelevantIndicesAndStoreLocally(relevantIndices, respParamsStr, dbFieldObj, results, local_db_counter, QueryDetails);
                        QueryDetails.strSql += ' WHERE record_date=' + rec_date_int + ';';

                        // Advance to next element in database.  Loop guards against duplicate entries (even though that shouldn't be possible by design)
                        last_rec_date_int = rec_date_int;
                        while ((local_db_counter < len) && (last_rec_date_int == rec_date_int))
                        {
                            local_db_counter += 1;
                            if (local_db_counter < len)
                            {
                                rec_date_int = results.rows.item(local_db_counter)['record_date'];
                            }
                        }
                        local_db_match = 1;                                                                
                    }
                }
                // Just create a new database entry if one didn't already exist
                if (local_db_match == 0)
                {
                    // Make sure the server returned something by checking a couple of parameters we will put in the local database
                    if (respParamsStr[index2_Color_field[i]] != '')
                    {         
                        var strSql_fields;
                        var strSql_values;    
                        
                        // No match.  We must create a database entry
                        strSql_fields = 'INSERT INTO Interp_DB (record_date, color, cd, ';
                        strSql_values = 'VALUES (' + parseInt(start_date_as_int+i-1) + ', "' + respParamsStr[index2_Color_field[i]] + '", ' + respParamsStr[index2_Cd_field[i]] + ', ';
                        
                        strSql_fields += 'dpo, preg_cnt, rec_test, bbt_ov_cl, mon_ov, cm_ov, ';
                        strSql_values += respParamsStr[index2_Dpo_field[i]] + ', ' + respParamsStr[index2_PregCnt_field[i]] + ', ' + respParamsStr[index2_RecTest_field[i]] + ', ' + respParamsStr[index2_BBTOvCLen_field[i]] + ', ' + respParamsStr[index2_MonOv_field[i]] + ', ' + respParamsStr[index2_CMOv_field[i]] + ', ';
                        
                        strSql_fields += 'all_ov, fwin, fpct_all, fpct_cm, fpct_saliva, fpct_mon, ';
                        strSql_values += respParamsStr[index2_AllOv_field[i]] + ', ' + respParamsStr[index2_FWin_field[i]] + ', ' + respParamsStr[index2_FPctAll_field[i]] + ', ' + respParamsStr[index2_FPctCm_field[i]] + ', ' + respParamsStr[index2_FPctSaliva_field[i]] + ', ' + respParamsStr[index2_FPctMon_field[i]] + ', ';
                        
                        if (BUILD_TARGET == BUILD_4_CCL)
                        {
                            strSql_fields += 'bill_itp, cclst_cov, ';
                            strSql_values += '"' + respParamsStr[index2_BillInterp_field[i]] + '", ' + respParamsStr[index2_CclStCov_field[i]] + ', ';
                        }
                        
                        strSql_fields += 'fpct_bbt, fpct_rhy, fpct_lh, fpct_cervix) ';
                        strSql_values += respParamsStr[index2_FPctBbt_field[i]] + ', ' + respParamsStr[index2_FPctRhy_field[i]] + ', ' + respParamsStr[index2_FPctLh_field[i]] + ', ' + respParamsStr[index2_FPctCervix_field[i]] + ');';

                        QueryDetails.strSql = strSql_fields + strSql_values;
                        QueryDetails.execute_strSql = 1;
                    }
                }
                // Execute query
                if (QueryDetails.execute_strSql != 0)
                {
                    // Note: suspend_scrolling(true, '') already called above in a blanket call for States 7 thru 14
                    tx.executeSql(QueryDetails.strSql, [], successCB, errorCB);
                }
            }
            
            // Special considerations if the user is not an Active Subscriber
            if (global_active_subscription == 0) 
            {
                WarnIfNoActiveSubscription();
                if (targetCalendar_OR_Analysis_NOT)
                {
                    // Make sure the shading legend never shows up because it's not supported for non-subscribers
                    document.getElementById('shaded_cal_legend').style.display = 'none';
                    if (BUILD_TARGET == BUILD_4_CCL)
                    {
                        document.getElementById('ccl_legend_nosub').style.display = 'block';
                    }
                    else
                    {
                        if (global_is_ios)
                        {
                            document.getElementById('nosub_cal_legend_ios').style.display = 'block';
                        }
                        else
                        {
                            document.getElementById('nosub_cal_legend').style.display = 'block';
                        }
                    }
                }
            }
            else
            {
                // Must be an active subscriber.  We can clear this since this box is used to indicate subscription status issues or remote
                // connectivity issues (and we are far enough along to know we are doing a remote transfer so that must be enabled)
                clearInfoMessage();
            }
            
            //////////////////////////////////////////////
            // Part 1 of parallel execution is finished //
            //////////////////////////////////////////////
            LoadCalScreen_InterpDB = 1;
        
            // Check if all parallel parts are complete.  If so, complete by executing the callback
            if ((LoadCalScreen_InterpDB == 1) && (LoadCalScreen_DataRecsDB == 1) && (LoadCalScreen_EventSettingsDB == 1) && (LoadCalScreen_CyclesDB == 1) && (LoadCalScreen_UserInfoDB == 1) && (LoadCalScreen_PregDB == 1))
            {
                // Indicate that this will take ownership of executing the callback.  Part 1 has highest priority, followed by Part 2 and part 3 respectively
                LoadCalScreen_InterpDB = 0;
                
                // Double check that another parallel task (of higher priority) didn't take ownership by clearing its flag.  We don't want to have more than
                // one parallel thread executing the callback.
                // NOTE: Part 1 always has priority and doesn't need to check for ties.  It always wins if there is a conflict (the later "parts" defer to it)
                
                // Timestamp callback
                var currTime = new Date();
                printDebug("State #" + state + " of data_load_fsm parallelism done " + appendTimeStamp(currTime));
                
                // Continue execution after parallel execution paths
                data_load_fsm(LCS_AFTER_PARALLEL_EXEC_STATE_14, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, '', '');              
            }
            break;     
            
        case LCS_PART_2_STATE_9:
LogCall('a9');
            ///////////////////////////////////////
            // PART 2 - Compare with Data_Recs_DB //
            ///////////////////////////////////////
        
            // Analyze server response and update local database with necessary changes (Interp_DB)
            var desiredStrStart=new Array();        // Start delimeter to look for in server response
            var desiredStrEnd=new Array();          // End delimeter to look for in server response
            var dbFieldObj=new Array();             // Pointers to objects describing database fields
            var respParamsStr=new Array();          // Placeholder for the parsed response found in the server returned content.
            
            var index2_Mens_field=new Array();
            var index2_Temp_field=new Array();
            var index2_Unsure_field=new Array();
            var index2_Discard_field=new Array();
            var index2_Mucus_field=new Array();
            var index2_Monitor_field=new Array();
            var index2_OvTestAm_field=new Array();
            var index2_OvTestPm_field=new Array();
            var index2_Saliva_field=new Array();
            var index2_Cervix_field=new Array();
            var index2_INotes_field=new Array();
            var index2_PNotes_field=new Array();
            var index2_TNotes_field=new Array();
            var index2_Events_1_field=new Array();
            var index2_Events_2_field=new Array();
            var index2_Events_3_field=new Array();
            var index2_Events_4_field=new Array();
            var index2_FWeight_field=new Array();
            var index2_MFlow_field=new Array();
            var index2_CervCons_field=new Array();
            var index2_CervDia_field=new Array();
            var index2_MucusSense_field=new Array();
            var index2_MucusCons_field=new Array();
            var str_without_quotes;
            
            var newContent = param_3;
            var local_db_counter = 0;
            var len = results.rows.length;
            var rec_date_int;   
            var strSql_fields;
            var strSql_values;
            
            // Prepare variables to extract from response                    
            for (var i=1; i<=number_of_days; i++)
            {
				index2_Temp_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='TEMP' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbDataDBField_Temp;
				
				index2_Mens_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='MENS' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbDataDBField_Mens;
				
				index2_Unsure_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='UNSURE' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbDataDBField_Unsure;
				
				index2_Discard_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='DISCARD' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbDataDBField_Discard;
				
				index2_OvTestAm_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='OVTEST_AM' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbDataDBField_OvTestAm;
                
                index2_OvTestPm_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='OVTEST_PM' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbDataDBField_OvTestPm;
				
				index2_Monitor_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='MONITOR' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbDataDBField_Monitor;

                if (BUILD_TARGET == BUILD_4_CCL)
                {
                    index2_FWeight_field[i] = desiredStrStart.length;
                    desiredStrStart[desiredStrStart.length]='FWEIGHT' + i + '=';
                    desiredStrEnd[desiredStrEnd.length]=" ";
                    dbFieldObj[dbFieldObj.length]=dbDataDBField_Weight;
                    
                    index2_MFlow_field[i] = desiredStrStart.length;
                    desiredStrStart[desiredStrStart.length]='MFLOW' + i + '=';
                    desiredStrEnd[desiredStrEnd.length]=" ";
                    dbFieldObj[dbFieldObj.length]=dbDataDBField_MFlow;
                    
                    index2_CervCons_field[i] = desiredStrStart.length;
                    desiredStrStart[desiredStrStart.length]='CERV_CONS' + i + '=';
                    desiredStrEnd[desiredStrEnd.length]=" ";
                    dbFieldObj[dbFieldObj.length]=dbDataDBField_CervCons;
                    
                    index2_CervDia_field[i] = desiredStrStart.length;
                    desiredStrStart[desiredStrStart.length]='CERV_DIA' + i + '=';
                    desiredStrEnd[desiredStrEnd.length]=" ";
                    dbFieldObj[dbFieldObj.length]=dbDataDBField_CervDia;
                    
                    index2_MucusSense_field[i] = desiredStrStart.length;
                    desiredStrStart[desiredStrStart.length]='MUC_SENS' + i + '=';
                    desiredStrEnd[desiredStrEnd.length]=" ";
                    dbFieldObj[dbFieldObj.length]=dbDataDBField_MucusSense;
                    
                    index2_MucusCons_field[i] = desiredStrStart.length;
                    desiredStrStart[desiredStrStart.length]='MUC_ACONS' + i + '=';
                    desiredStrEnd[desiredStrEnd.length]=" ";
                    dbFieldObj[dbFieldObj.length]=dbDataDBField_MucusAcons;
                    
                    // This field is upload only. It makes a request to set a cycle ID and that request gets cleared 
                    //index2_IdTxtReq_field[i] = desiredStrStart.length;
                    //desiredStrStart[desiredStrStart.length]='ITXTREQ' + i + '=';
                    //desiredStrEnd[desiredStrEnd.length]=" ";
                    //dbFieldObj[dbFieldObj.length]=dbDataDBField_IdTxtReq;                    
                }
                else
                {
                    index2_Saliva_field[i] = desiredStrStart.length;
                    desiredStrStart[desiredStrStart.length]='SALIVA' + i + '=';
                    desiredStrEnd[desiredStrEnd.length]=" ";
                    dbFieldObj[dbFieldObj.length]=dbDataDBField_Saliva;
                    
                    index2_Cervix_field[i] = desiredStrStart.length;
                    desiredStrStart[desiredStrStart.length]='CERVIX' + i + '=';
                    desiredStrEnd[desiredStrEnd.length]=" ";
                    dbFieldObj[dbFieldObj.length]=dbDataDBField_Cervix;
                }
                
                index2_Mucus_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='MUCUS' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbDataDBField_Mucus;
			}
			for (var i=1; i<=number_of_days; i++)
            {
				index2_Events_1_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='EV1NT' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbDataDBField_Events_1;
                
                index2_Events_2_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='EV2NT' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbDataDBField_Events_2;
                
                index2_Events_3_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='EV3NT' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbDataDBField_Events_3;
                
                index2_Events_4_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='EV4NT' + i + '=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbDataDBField_Events_4;
			}
			for (var i=1; i<=number_of_days; i++)
            {
				index2_PNotes_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='P_NOTES' + i + '=';
                desiredStrEnd[desiredStrEnd.length]="@!@";
                dbFieldObj[dbFieldObj.length]=dbDataDBField_PNotes;
				
				index2_INotes_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='I_NOTES' + i + '=';
                desiredStrEnd[desiredStrEnd.length]="@!@";
                dbFieldObj[dbFieldObj.length]=dbDataDBField_INotes;
                
                index2_TNotes_field[i] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='T_NOTES' + i + '=';
                desiredStrEnd[desiredStrEnd.length]="@!@";
                dbFieldObj[dbFieldObj.length]=dbDataDBField_TNotes;
			}

            // Parse the response to find the desired values
            respParamsStr = parse_server_response(newContent, desiredStrStart, desiredStrEnd);
               
            // Loop through the days spanning the range
            for (var i=1; i<=number_of_days; i++)
            {
                local_db_match = 0;
                var QueryDetails = {execute_strSql: 0, strSql: ''};
                
                // See if the local database already has a value that matches this
                if (local_db_counter < len)
                {
                    rec_date_int = results.rows.item(local_db_counter)['record_date'];
                    if ((rec_date_int - start_date_as_int + 1) == i)
                    {
                        // This is a match.  We must update the existing database entry and consider "_chg" flag values
                        QueryDetails.strSql = 'UPDATE Data_Recs_DB SET ';
                        
                        // Create a list of database elements we need to consider
                        var given_index;
                        var last_read_dirty_prefix = '';
                        var last_read_chg = '';
                        var last_read_pnd = '';
                        var relevantIndices=new Array();                                
                        relevantIndices[relevantIndices.length] = index2_Mens_field[i];
                        relevantIndices[relevantIndices.length] = index2_Temp_field[i];
                        relevantIndices[relevantIndices.length] = index2_Unsure_field[i];
                        relevantIndices[relevantIndices.length] = index2_Discard_field[i];
                        relevantIndices[relevantIndices.length] = index2_Mucus_field[i];
                        relevantIndices[relevantIndices.length] = index2_Monitor_field[i];
                        relevantIndices[relevantIndices.length] = index2_OvTestAm_field[i];
                        relevantIndices[relevantIndices.length] = index2_OvTestPm_field[i];
                        if (BUILD_TARGET == BUILD_4_CCL)
                        {
                            relevantIndices[relevantIndices.length] = index2_FWeight_field[i];
							relevantIndices[relevantIndices.length] = index2_MFlow_field[i];
                            relevantIndices[relevantIndices.length] = index2_CervCons_field[i];
                            relevantIndices[relevantIndices.length] = index2_CervDia_field[i];
                            relevantIndices[relevantIndices.length] = index2_MucusSense_field[i];
                            relevantIndices[relevantIndices.length] = index2_MucusCons_field[i];
                        }
                        else
                        {
                            relevantIndices[relevantIndices.length] = index2_Saliva_field[i];
                            relevantIndices[relevantIndices.length] = index2_Cervix_field[i];
                        }
                        relevantIndices[relevantIndices.length] = index2_INotes_field[i];
                        relevantIndices[relevantIndices.length] = index2_PNotes_field[i];
                        relevantIndices[relevantIndices.length] = index2_TNotes_field[i];
                        relevantIndices[relevantIndices.length] = index2_Events_1_field[i];
                        relevantIndices[relevantIndices.length] = index2_Events_2_field[i];
                        relevantIndices[relevantIndices.length] = index2_Events_3_field[i];
                        relevantIndices[relevantIndices.length] = index2_Events_4_field[i];
                        
                        // Call a function to walk through the relevantIndices and build the appropriate query string and indicate if it should execute using QueryDetails
                        QueryDetails = loopThruRelevantIndicesAndStoreLocally(relevantIndices, respParamsStr, dbFieldObj, results, local_db_counter, QueryDetails);
                        QueryDetails.strSql += ' WHERE record_date=' + rec_date_int + ';';
                        
                        // Advance to next element in database.  Loop guards against duplicate entries (even though that shouldn't be possible by design)
                        last_rec_date_int = rec_date_int;
                        while ((local_db_counter < len) && (last_rec_date_int == rec_date_int))
                        {
                            local_db_counter += 1;
                            if (local_db_counter < len)
                            {
                                rec_date_int = results.rows.item(local_db_counter)['record_date'];
                            }
                        }
                        local_db_match = 1;                                                                
                    }
                }
                // Just create a new database entry if one didn't already exist
                if (local_db_match == 0)
                {
                    // Make sure the server returned something by checking a couple of parameters we will put in the local database
                    if ((respParamsStr[index2_Mens_field[i]] != '') && (respParamsStr[index2_Temp_field[i]] != ''))
                    {
                        // No match.  We must create a database entry
                        strSql_fields = 'INSERT INTO Data_Recs_DB (record_date, ';
                        strSql_values = 'VALUES (' + parseInt(start_date_as_int+i-1) + ', ';
                        
                        strSql_fields += 'fdata_chg, fdata_pnd, ';
                        strSql_values += '0, 0, ';
                        
                        strSql_fields += 'mens, i_notes, ';
                        strSql_values += respParamsStr[index2_Mens_field[i]] + ', "' + ReplaceQuotes(respParamsStr[index2_INotes_field[i]], 1) + '", ';
                        
                        strSql_fields += 'p_notes, temp, unsure, discard, ';
                        strSql_values += '"' + ReplaceQuotes(respParamsStr[index2_PNotes_field[i]],1) + '", ' + respParamsStr[index2_Temp_field[i]] + ', ' + respParamsStr[index2_Unsure_field[i]] + ', ' + respParamsStr[index2_Discard_field[i]] + ', ';
                        
                        strSql_fields += 't_notes, ';
                        strSql_values += '"' + ReplaceQuotes(respParamsStr[index2_TNotes_field[i]], 1) + '", ';
                        
                        strSql_fields += 'mucus, ';
                        strSql_values += '"' + respParamsStr[index2_Mucus_field[i]] + '", ';
                        
                        if (BUILD_TARGET == BUILD_4_CCL)
                        {
                            strSql_fields += 'fweight, ';
                            strSql_values += '"' + respParamsStr[index2_FWeight_field[i]] + '", ';
                        
                            strSql_fields += 'mflow, cerv_cons, cerv_dia, muc_sens,  muc_acons, itxtreq, ';
                            strSql_values += '"' + respParamsStr[index2_MFlow_field[i]] + '", "' + respParamsStr[index2_CervCons_field[i]] + '", "' + respParamsStr[index2_CervDia_field[i]] + '", "' + respParamsStr[index2_MucusSense_field[i]] + '", "' + respParamsStr[index2_MucusCons_field[i]] + '", 0, ';
                        }
                        else
                        {
                            strSql_fields += 'cervix, saliva, ';
                            strSql_values += '"' + respParamsStr[index2_Cervix_field[i]] + '", "' + respParamsStr[index2_Saliva_field[i]] + '", ';
                        }
                        
                        strSql_fields += 'monitor, ovtest_am, ovtest_pm';    
                        strSql_values += '"' + respParamsStr[index2_Monitor_field[i]] + '", "' + respParamsStr[index2_OvTestAm_field[i]] + '", "' + respParamsStr[index2_OvTestPm_field[i]] + '"'; 
                        
                        for (var k=1; k<=NUMBER_OF_EVENT_WORDS; k++)
                        {
                            strSql_fields += ', ev' + k + 'nt, ev' + k + 'nt_chg, ev' + k + 'nt_pnd';
                        }
                        strSql_values += ', ' + respParamsStr[index2_Events_1_field[i]] + ', 0, 0';
                        strSql_values += ', ' + respParamsStr[index2_Events_2_field[i]] + ', 0, 0';
                        strSql_values += ', ' + respParamsStr[index2_Events_3_field[i]] + ', 0, 0';
                        strSql_values += ', ' + respParamsStr[index2_Events_4_field[i]] + ', 0, 0';
                        
                        strSql_fields += ', mkr) ';
                        strSql_values += ', 0);';

                        QueryDetails.strSql = strSql_fields + strSql_values;
                        QueryDetails.execute_strSql = 1;
                    }
                }
                // Execute query
                if (QueryDetails.execute_strSql != 0)
                {
                    // Note: suspend_scrolling(true, '') already called above in a blanket call for States 7 thru 14
                    tx.executeSql(QueryDetails.strSql, [], successCB, errorCB);
                }
            }
            
            //////////////////////////////////////////////
            // Part 2 of parallel execution is finished //
            //////////////////////////////////////////////
            LoadCalScreen_DataRecsDB = 1;
        
            // Check if all parallel parts are complete.  If so, complete by executing the callback
            if ((LoadCalScreen_InterpDB == 1) && (LoadCalScreen_DataRecsDB == 1) && (LoadCalScreen_EventSettingsDB == 1) && (LoadCalScreen_CyclesDB == 1) && (LoadCalScreen_UserInfoDB == 1) && (LoadCalScreen_PregDB == 1))
            {
                // Indicate that this will take ownership of executing the callback.  Part 1 has highest priority, followed by Part 2 and part 3 respectively
                LoadCalScreen_DataRecsDB = 0;
                
                // Delay for a moment and check again that a higher priority parallel path didn't already declare ownership of doing the callback
                for (var x=0; x<5; x++)
                {
                    var i=x+2;
                }
                
                // Double check that another parallel task (of higher priority) didn't take ownership by clearing its flag.  We don't want to have more than
                // one parallel thread executing the callback.
                if (LoadCalScreen_InterpDB == 1)
                {
                    // Timestamp callback
                    var currTime = new Date();
                    printDebug("State #" + state + " of data_load_fsm parallelism done " + appendTimeStamp(currTime));
                    
                    // Continue execution after parallel execution paths
                    data_load_fsm(LCS_AFTER_PARALLEL_EXEC_STATE_14, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, '', '');
                }              
            }
            break;     
            
        case LCS_PART_3_STATE_10:
LogCall('aa');
            /////////////////////////////////////////////
            // PART 3 - Compare with Event_Settings_DB //
            /////////////////////////////////////////////
        
            // Analyze server response and update local database with necessary changes (Interp_DB)
            var desiredStrStart=new Array();        // Start delimeter to look for in server response
            var desiredStrEnd=new Array();          // End delimeter to look for in server response
            var dbFieldObj=new Array();             // Pointers to objects describing database fields
            var respParamsStr=new Array();          // Placeholder for the parsed response found in the server returned content.
            
            var index2_EventName_field=new Array();
            
            var newContent = param_3;
            var local_db_counter = 0;
            var len = results.rows.length;
            var rec_date_int;
            
            // Check that the local database query worked.  If it doesn't, display an error and exit
            if (len == 0)
            {
                printInfoMessage('Error re-querying local EventSettings database.  Please try again.  If this problem persists, you may want to try going to the settings page and re-associating with your account.');
                break;
            }

            // Prepare variables to extract from response                    
            index2_EventName_field[0] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_1=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_1;  

            index2_EventName_field[1] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_2=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_2; 

            index2_EventName_field[2] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_3=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_3; 

            index2_EventName_field[3] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_4=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_4; 

            index2_EventName_field[4] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_5=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_5; 

            index2_EventName_field[5] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_6=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_6; 

            index2_EventName_field[6] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_7=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_7; 

            index2_EventName_field[7] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_8=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_8; 

            index2_EventName_field[8] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_9=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_9; 

            index2_EventName_field[9] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_10=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_10; 

            index2_EventName_field[10] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_11=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_11; 

            index2_EventName_field[11] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_12=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_12; 

            index2_EventName_field[12] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_13=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_13; 

            index2_EventName_field[13] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_14=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_14; 

            index2_EventName_field[14] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_15=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_15; 

            index2_EventName_field[15] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_16=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_16; 

            index2_EventName_field[16] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_17=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_17; 

            index2_EventName_field[17] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_18=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_18; 

            index2_EventName_field[18] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_19=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_19; 

            index2_EventName_field[19] = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'CEV_NAME_20=';
            desiredStrEnd[desiredStrEnd.length]="@!@";
            dbFieldObj[dbFieldObj.length]=dbEvSetDBField_EvName_20;             
            
            // Parse the response to find the desired values
            respParamsStr = parse_server_response(newContent, desiredStrStart, desiredStrEnd);
               
            // There is one and only 1 record in the local database, so we don't need to go looking for a match like with other databases.  
            local_db_match = 0;
            var QueryDetails = {execute_strSql: 0, strSql: ''};

            // Update the existing database entry and consider "_chg" flag values
            QueryDetails.strSql = 'UPDATE Event_Settings_DB SET ';
            
            // Create a list of database elements we need to consider
            var given_index;
            var last_read_dirty_prefix = '';
            var last_read_chg = '';
            var last_read_pnd = '';
            var relevantIndices=new Array();  
            for (j=0; j<index2_EventName_field.length; j++)
            {
                relevantIndices[relevantIndices.length] = index2_EventName_field[j];
            }
            
            // Call a function to walk through the relevantIndices and build the appropriate query string and indicate if it should execute using QueryDetails
            QueryDetails = loopThruRelevantIndicesAndStoreLocally(relevantIndices, respParamsStr, dbFieldObj, results, local_db_counter, QueryDetails);
            QueryDetails.strSql += ' WHERE id=' + results.rows.item(local_db_counter)['id'] + ';';

            // Execute query
            if (QueryDetails.execute_strSql != 0)
            {
                // Note: suspend_scrolling(true, '') already called above in a blanket call for States 7 thru 14
                tx.executeSql(QueryDetails.strSql, [], successCB, errorCB);
            }
            
            //////////////////////////////////
            // Check for a Server Info Message
            //////////////////////////////////
            desiredStrStart = new Array();        // Start delimeter to look for in server response
            desiredStrEnd = new Array();          // End delimeter to look for in server response
            
            var survey_index = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'SURVEY=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            
            var rate_us_index = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'RATE_US=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            
            var svr_info_index = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]= 'SERVER_INFO=';
            desiredStrEnd[desiredStrEnd.length]="@!@";

            // Parse the response to find the desired values
            respParamsStr = parse_server_response(newContent, desiredStrStart, desiredStrEnd);
            
            // Check if told to solicit for review
            if ((respParamsStr[survey_index] != 0) && (isNaN(respParamsStr[survey_index]) == false))
            {
                global_survey_id = parseInt(respParamsStr[survey_index]);
                strSql = 'UPDATE User_Info_DB SET survey_id=' + global_survey_id + ' WHERE 1';
                tx.executeSql(strSql, [], successCB, errorCB);
            }

            // Update the local database if something was returned
            if ((global_svr_info_msg != respParamsStr[svr_info_index]) || (respParamsStr[rate_us_index] != 0))
            {
                var shown_state = 1;
                var suppress_inserting_info_msg = false;
                var update_db = true;
                
                // svr_info_msg_shown = 0 means the message needs shown (server info message)
                // svr_info_msg_shown = 1 means the message was already shown and need not be shown again (server info message)
                // svr_info_msg_shown = 10 means the Rate Our App ad #1 should be run (incentive version)
                //                    = 11 means it was successfully shown
                // svr_info_msg_shown = 20 means the Rate Our App ad #2 should be run (non-incentive version)
                //                    = 21 means it was successfully shown
                if (respParamsStr[rate_us_index] != 0)
                {
                    // Need to display Rate Our App ad unless we already did
                    if (global_svr_info_msg_shown != (parseInt(respParamsStr[rate_us_index]) + 1))
                    {
                        shown_state = respParamsStr[rate_us_index];
                    }
                    else
                    {
                        // We already displayed this ad and don't want to do it again, so don't touch the database
                        update_db = false;
                    }
                    // If coincidentally the server also is requesting a new Info message at the same time, hold off saving it until later so it will still get shown later
                    if (global_svr_info_msg != respParamsStr[svr_info_index])
                    {
                        suppress_inserting_info_msg = true;
                    }
                }
                else if (respParamsStr[svr_info_index] != '')
                {
                    // Need to display server message
                    shown_state = 0;
                }
                strSql = 'UPDATE User_Info_DB SET ';
                if (suppress_inserting_info_msg == false)
                {
                    strSql += 'svr_info_msg="' + respParamsStr[svr_info_index] + '", ';
                }
                strSql += 'svr_info_msg_shown=' + shown_state + ' WHERE 1';
                if (update_db == true)
                {
                    // Note: suspend_scrolling(true, '') already called above in a blanket call for States 7 thru 14
                    tx.executeSql(strSql, [], successCB, errorCB);
                    global_svr_info_msg = respParamsStr[svr_info_index];
                    global_svr_info_msg_shown = shown_state;
                }
            }

            //////////////////////////////////////////////
            // Part 3 of parallel execution is finished //
            //////////////////////////////////////////////
            LoadCalScreen_EventSettingsDB = 1;
            var currTime = new Date(); 
            g_LoadTimestamps['events'] = currTime.getTime();        // Timestamp events so there's no need to reload them again too soon

            // Check if all parallel parts are complete.  If so, complete by executing the callback
            if ((LoadCalScreen_InterpDB == 1) && (LoadCalScreen_DataRecsDB == 1) && (LoadCalScreen_EventSettingsDB == 1) && (LoadCalScreen_CyclesDB == 1) && (LoadCalScreen_UserInfoDB == 1) && (LoadCalScreen_PregDB == 1))
            {
                // Indicate that this will take ownership of executing the callback.  Part 1 has highest priority, followed by Part 2 and part 3 respectively
                LoadCalScreen_EventSettingsDB = 0;
                
                // Delay for a moment and check again that a higher priority parallel path didn't already declare ownership of doing the callback
                for (var x=0; x<12; x++)
                {
                    var i=x+2;
                }
                
                // Double check that another parallel task (of higher priority) didn't take ownership by clearing its flag.  We don't want to have more than
                // one parallel thread executing the callback.
                if ((LoadCalScreen_InterpDB == 1) && (LoadCalScreen_DataRecsDB == 1))
                {
                    // Timestamp callback
                    var currTime = new Date();
                    printDebug("State #" + state + " of data_load_fsm parallelism done " + appendTimeStamp(currTime));
                
                    // Continue execution after parallel execution paths
                    data_load_fsm(LCS_AFTER_PARALLEL_EXEC_STATE_14, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, '', '');
                }              
            }
            break;    

        case LCS_PART_4_STATE_11:
LogCall('ab');
            ///////////////////////////////////////
            // PART 4 - Compare with Cycles_DB //    -- This only gets data for the actual cycle.  Cycle stats crossing cycles are queried when the drop down is looked up
            ///////////////////////////////////////
        
            // Analyze server response and update local database with necessary changes (Interp_DB)
            var desiredStrStart=new Array();        // Start delimeter to look for in server response
            var desiredStrEnd=new Array();          // End delimeter to look for in server response
            var dbFieldObj=new Array();             // Pointers to objects describing database fields
            var respParamsStr=new Array();          // Placeholder for the parsed response found in the server returned content.

            var str_without_quotes;
            
            var newContent = param_3;
            var local_db_counter = 0;
            var len = results.rows.length;
            var rec_date_int;  

            // Prepare variables to extract from response
            desiredStrStart[desiredStrStart.length]='TOTAL_CYC=';
            desiredStrEnd[desiredStrEnd.length]=" ";

            // Initially parse the response to find the number of cycles
            respParamsStr = parse_server_response(newContent, desiredStrStart, desiredStrEnd);
            var num_cycles = respParamsStr[0];
          
            // Prepare variables to extract from response for all returned cycles
            desiredStrStart.length = 0;
            desiredStrEnd.length = 0;            

            // Prepare variables to extract from response   
            index2_CycLen_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='CYC_LEN=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesDBField_CycLen;

            index2_CycLutLen_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='CYC_LUT_LEN=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesDBField_LutLen;

            index2_CycOvCd_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='CYC_OV_CD=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesDBField_OvCd;

            index2_CycOvType_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='CYC_OV_TYPE=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_AllOvType;
            
            index2_AllOvMan_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='ALL_OV_MANUAL=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_AllOvManual;
                        
            index2_AllOvAuto_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='ALL_OV_AUTO=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesDBField_AllOvAuto;
            
            index2_Anov_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='ANOV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_Anov;
            
            index2_AllOvType_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='ALL_OV_TYPE=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_AllOvType;

            index2_LhOv_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='LH_OV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_LhOv; 
            
            index2_MonOv_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='MON_OV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_MonOv; 
            
            index2_RhyOv_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='RHY_OV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_RhyOv; 

            index2_CmOv_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='CM_OV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_CmOv;  
            
            index2_CmTradOv_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='CM_TRAD_OV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_CmTradOv;  
            
            index2_CmAltOv_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='CM_ALT_OV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_CmAltOv;  
            
            index2_CmMeth_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='CM_METH=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_CmMeth;
            
            index2_BbtOv_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]=' BBT_OV=';  // needs extra leading space to distinguish from EARLY_BBT_OV
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesDBField_BbtOv;
            
            index2_SalivaOv_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='SALIVA_OV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_SalivaOv; 
            
            index2_BbtCvOv_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='BBT_CV_OV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesDBField_BbtCvOv;
            
            index2_BbtAveOv_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='BBT_AVE_OV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesDBField_BbtAveOv;
            
            index2_BbtSmcOv_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='BBT_SMC_OV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesDBField_BbtSmOv;
            
            index2_BbtOrigOv_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='BBT_ORIG_OV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesDBField_BbtOrigOv;

            index2_Bbt36Ov_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='BBT_36_OV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesDBField_Bbt36Ov;

            index2_BbtMeth_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='BBT_METH=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_BbtMeth;
           
            if (BUILD_TARGET == BUILD_4_CCL)
            {
                index2_FwbSt_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FWB_ST=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_FwbSt;
                
                index2_FweSt_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FWE_ST=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_FweSt;
                
                index2_InstNotes_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='INST_NOTES=';
                desiredStrEnd[desiredStrEnd.length]="@!@.";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_InstNotes;
                
                index2_IdTxt_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='ID_TXT=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_IdTxt; 

                index2_FwbIn_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FWB_IN=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_FwbIn;
                
                index2_FweIn_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FWE_IN=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_FweIn;
                
                index2_FwbManIn_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FWB_MAN_IN=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_FwbManIn;
                
                index2_FweManIn_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FWE_MAN_IN=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_FweManIn;
                
                index2_FwbManSt_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FWB_MAN_ST=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_FwbManSt;
                
                index2_FweManSt_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FWE_MAN_ST=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_FweManSt;
                
                index2_FwbVals_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FWB_VALS=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_FwbVals;
                
                index2_FweVals_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='FWE_VALS=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_FweVals;
                
                index2_CycWeight_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='CWEIGHT=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_Weight;
                
                index2_CycNotes_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='CYC_NOTES=';
                desiredStrEnd[desiredStrEnd.length]="@!@.";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_CycNotes;
            }

            index2_Certain_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='CERTAIN=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_Certain; 
            
            index2_Quality_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='QUALITY=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_Quality;

            index2_Complete_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='COMPLETE=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_Complete; 

            index2_FWinBeg_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='FWIN_BEG=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_FWinBeg;      

            index2_FWinEnd_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='FWIN_END=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_FWinEnd;   

            index2_FWinEndFinal_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='FWIN_END_FINAL=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_FWinEndFinal;   

            index2_FWinBegFinal_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='FWIN_BEG_FINAL=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_FWinBegFinal;   
            
            index2_FWinWhy_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='FWIN_WHY=';
            desiredStrEnd[desiredStrEnd.length]="@!@.";
            dbFieldObj[dbFieldObj.length]=dbCyclesBField_FWinWhy;                   

            if (BUILD_TARGET == BUILD_4_CCL)
            {
                index2_PcShort_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='PC_SHORT=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_PcShort;
                
                index2_PcLong_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='PC_LONG=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_PcLong;
                
                index2_PcNumCyc_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='PC_NUM_CYC=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_PcNumCyc;
                
                index2_PcEarlyTrise_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='PC_EARLY_TRISE=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_PcEarlyTrise;
                
                index2_PcNumTcyc_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='PC_NUM_TCYC=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_PcNumTcyc;
                
                index2_PCLongMucPatches_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='PC_L_MUCPAT=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbCyclesBField_PcLongMucPatches;
            }

            // Parse the response to find the desired values
            respParamsStr = parse_server_response(newContent, desiredStrStart, desiredStrEnd);

            // Convert missing fields to neutral values (likely missing due to being a non-subscriber)
            respParamsStr[index2_CycOvType_field] = IfEmptyAssignAlternate(respParamsStr[index2_CycOvType_field], 1);
            respParamsStr[index2_CycLen_field] = IfEmptyAssignAlternate(respParamsStr[index2_CycLen_field], 0);
            respParamsStr[index2_Anov_field] = IfEmptyAssignAlternate(respParamsStr[index2_Anov_field], 0);
            respParamsStr[index2_CycOvCd_field] = IfEmptyAssignAlternate(respParamsStr[index2_CycOvCd_field], 0);
            respParamsStr[index2_BbtMeth_field] = IfEmptyAssignAlternate(respParamsStr[index2_BbtMeth_field], 0);
            respParamsStr[index2_CmMeth_field] = IfEmptyAssignAlternate(respParamsStr[index2_CmMeth_field], 0);
            if (BUILD_TARGET == BUILD_4_CCL)
            {
                respParamsStr[index2_CycWeight_field] = IfEmptyAssignAlternate(respParamsStr[index2_CycWeight_field], 0);
                respParamsStr[index2_CycNotes_field] = IfEmptyAssignAlternate(respParamsStr[index2_CycNotes_field], '');
                respParamsStr[index2_FwbSt_field] = IfEmptyAssignAlternate(respParamsStr[index2_FwbSt_field], 3);
                respParamsStr[index2_FweSt_field] = IfEmptyAssignAlternate(respParamsStr[index2_FweSt_field], 2);
                respParamsStr[index2_FwbIn_field] = IfEmptyAssignAlternate(respParamsStr[index2_FwbIn_field], 3);
                respParamsStr[index2_FweIn_field] = IfEmptyAssignAlternate(respParamsStr[index2_FweIn_field], 2);
                respParamsStr[index2_FwbManIn_field] = IfEmptyAssignAlternate(respParamsStr[index2_FwbManIn_field], 0);
                respParamsStr[index2_FweManIn_field] = IfEmptyAssignAlternate(respParamsStr[index2_FweManIn_field], 0);
                respParamsStr[index2_FwbManSt_field] = IfEmptyAssignAlternate(respParamsStr[index2_FwbManSt_field], 0);
                respParamsStr[index2_FweManSt_field] = IfEmptyAssignAlternate(respParamsStr[index2_FweManSt_field], 0);
                respParamsStr[index2_FwbVals_field] = IfEmptyAssignAlternate(respParamsStr[index2_FwbVals_field], '00000000000000000000000000');
                respParamsStr[index2_FweVals_field] = IfEmptyAssignAlternate(respParamsStr[index2_FweVals_field], '00000000000000');
                respParamsStr[index2_PcShort_field] = IfEmptyAssignAlternate(respParamsStr[index2_PcShort_field], 0);
                respParamsStr[index2_PcLong_field] = IfEmptyAssignAlternate(respParamsStr[index2_PcLong_field], 0);
                respParamsStr[index2_PcNumCyc_field] = IfEmptyAssignAlternate(respParamsStr[index2_PcNumCyc_field], 0);
                respParamsStr[index2_PcEarlyTrise_field] = IfEmptyAssignAlternate(respParamsStr[index2_PcEarlyTrise_field], 0);
                respParamsStr[index2_PcNumTcyc_field] = IfEmptyAssignAlternate(respParamsStr[index2_PcNumTcyc_field], 0);
                respParamsStr[index2_PCLongMucPatches_field] = IfEmptyAssignAlternate(respParamsStr[index2_PCLongMucPatches_field], 0);
                respParamsStr[index2_InstNotes_field] = IfEmptyAssignAlternate(respParamsStr[index2_InstNotes_field], '');
                respParamsStr[index2_IdTxt_field] = IfEmptyAssignAlternate(respParamsStr[index2_IdTxt_field], 0);
            }
            respParamsStr[index2_Certain_field] = IfEmptyAssignAlternate(respParamsStr[index2_Certain_field], -1);
            respParamsStr[index2_Quality_field] = IfEmptyAssignAlternate(respParamsStr[index2_Quality_field], -1);
            respParamsStr[index2_Complete_field] = IfEmptyAssignAlternate(respParamsStr[index2_Complete_field], -1);
            respParamsStr[index2_FWinBeg_field] = IfEmptyAssignAlternate(respParamsStr[index2_FWinBeg_field], '?');
            respParamsStr[index2_FWinEnd_field] = IfEmptyAssignAlternate(respParamsStr[index2_FWinEnd_field], '?');
            respParamsStr[index2_FWinBegFinal_field] = IfEmptyAssignAlternate(respParamsStr[index2_FWinBegFinal_field], 0);
            respParamsStr[index2_FWinEndFinal_field] = IfEmptyAssignAlternate(respParamsStr[index2_FWinEndFinal_field], 0);
            respParamsStr[index2_FWinWhy_field] = IfEmptyAssignAlternate(respParamsStr[index2_FWinWhy_field], 'Not Available');

            // Only one record is involved, so there is no need for looping
            local_db_match = 0;
            var QueryDetails = {execute_strSql: 0, strSql: ''};
            
            // See if the local database already has a value that matches this
            if (local_db_counter < len)
            {
                rec_date_int = results.rows.item(local_db_counter)['cyc_started'];
                if (rec_date_int == start_date_as_int)
                {
                    // This is a match.  We must update the existing database entry and consider "_chg" flag values
                    QueryDetails.strSql = 'UPDATE Cycles_DB SET ';
                    
                    // Create a list of database elements we need to consider
                    var given_index;
                    var last_read_dirty_prefix = '';
                    var last_read_chg = '';
                    var last_read_pnd = '';
                    var relevantIndices=new Array();                     
                    
                    relevantIndices[relevantIndices.length] = index2_CycLen_field;
                    relevantIndices[relevantIndices.length] = index2_CycLutLen_field;
                    relevantIndices[relevantIndices.length] = index2_CycOvCd_field;
                    relevantIndices[relevantIndices.length] = index2_CycOvType_field;
                    relevantIndices[relevantIndices.length] = index2_AllOvMan_field;
                    relevantIndices[relevantIndices.length] = index2_AllOvAuto_field;
                    relevantIndices[relevantIndices.length] = index2_Anov_field;
                    relevantIndices[relevantIndices.length] = index2_AllOvType_field;
                    relevantIndices[relevantIndices.length] = index2_LhOv_field;
                    relevantIndices[relevantIndices.length] = index2_MonOv_field;
                    relevantIndices[relevantIndices.length] = index2_RhyOv_field;
                    relevantIndices[relevantIndices.length] = index2_CmOv_field;
                    relevantIndices[relevantIndices.length] = index2_CmTradOv_field;
                    relevantIndices[relevantIndices.length] = index2_CmAltOv_field;
                    relevantIndices[relevantIndices.length] = index2_CmMeth_field;
                    relevantIndices[relevantIndices.length] = index2_BbtOv_field;
                    relevantIndices[relevantIndices.length] = index2_SalivaOv_field;
                    relevantIndices[relevantIndices.length] = index2_BbtCvOv_field;
                    relevantIndices[relevantIndices.length] = index2_BbtAveOv_field;
                    relevantIndices[relevantIndices.length] = index2_BbtSmcOv_field;
                    relevantIndices[relevantIndices.length] = index2_BbtOrigOv_field;
                    relevantIndices[relevantIndices.length] = index2_Bbt36Ov_field;
                    relevantIndices[relevantIndices.length] = index2_BbtMeth_field;
                    if (BUILD_TARGET == BUILD_4_CCL)
                    {
                        relevantIndices[relevantIndices.length] = index2_FwbSt_field;
                        relevantIndices[relevantIndices.length] = index2_FweSt_field;
                        relevantIndices[relevantIndices.length] = index2_InstNotes_field;
                        relevantIndices[relevantIndices.length] = index2_IdTxt_field;
                        relevantIndices[relevantIndices.length] = index2_FwbIn_field;
                        relevantIndices[relevantIndices.length] = index2_FweIn_field;
                        relevantIndices[relevantIndices.length] = index2_FwbManIn_field;
                        relevantIndices[relevantIndices.length] = index2_FweManIn_field;
                        relevantIndices[relevantIndices.length] = index2_FwbManSt_field;
                        relevantIndices[relevantIndices.length] = index2_FweManSt_field;
                        relevantIndices[relevantIndices.length] = index2_FwbVals_field;
                        relevantIndices[relevantIndices.length] = index2_FweVals_field;
                        relevantIndices[relevantIndices.length] = index2_CycWeight_field;
                        relevantIndices[relevantIndices.length] = index2_CycNotes_field;
                    }
                    relevantIndices[relevantIndices.length] = index2_Certain_field;
                    relevantIndices[relevantIndices.length] = index2_Quality_field;
                    relevantIndices[relevantIndices.length] = index2_Complete_field;
                    relevantIndices[relevantIndices.length] = index2_FWinBeg_field;
                    relevantIndices[relevantIndices.length] = index2_FWinEnd_field;
                    relevantIndices[relevantIndices.length] = index2_FWinEndFinal_field;
                    relevantIndices[relevantIndices.length] = index2_FWinBegFinal_field;
                    relevantIndices[relevantIndices.length] = index2_FWinWhy_field;  
                    if (BUILD_TARGET == BUILD_4_CCL)
                    {
                        relevantIndices[relevantIndices.length] = index2_PcShort_field;
                        relevantIndices[relevantIndices.length] = index2_PcLong_field;
                        relevantIndices[relevantIndices.length] = index2_PcNumCyc_field;
                        relevantIndices[relevantIndices.length] = index2_PcEarlyTrise_field;
                        relevantIndices[relevantIndices.length] = index2_PcNumTcyc_field;
                        relevantIndices[relevantIndices.length] = index2_PCLongMucPatches_field;
                    }                    
                    
                    // Call a function to walk through the relevantIndices and build the appropriate query string and indicate if it should execute using QueryDetails
                    QueryDetails = loopThruRelevantIndicesAndStoreLocally(relevantIndices, respParamsStr, dbFieldObj, results, local_db_counter, QueryDetails);
                    QueryDetails.strSql += ' WHERE cyc_started=' + rec_date_int + ';';
                    
                    // Advance to next element in database.  Loop guards against duplicate entries (even though that shouldn't be possible by design)
                    last_rec_date_int = rec_date_int;
                    while ((local_db_counter < len) && (last_rec_date_int == rec_date_int))
                    {
                        local_db_counter += 1;
                        if (local_db_counter < len)
                        {
                            rec_date_int = results.rows.item(local_db_counter)['record_date'];
                        }
                    }
                    local_db_match = 1;                                                                
                }
            }
            // Just create a new database entry if one didn't already exist
            if (local_db_match == 0)
            {
                // Make sure the server returned something by checking a couple of parameters we will put in the local database
                if ((respParamsStr[index2_AllOvAuto_field] != '') && (respParamsStr[index2_CycLen_field] != ''))
                {
                    var strSql_fields;
                    var strSql_values;
                    
                    // No match.  We must create a database entry. 
                    strSql_fields = 'INSERT INTO Cycles_DB (cyc_started, ';
                    strSql_values = 'VALUES (' + start_date_as_int + ', ';
                    
                    strSql_fields += 'cdata_chg, cdata_pnd, ';
                    strSql_values += '0, 0, ';
                    
                    strSql_fields += 'cyc_len, cyc_lut_len, anov, all_ov_manual, ';
                    strSql_values += '"' + respParamsStr[index2_CycLen_field] + '", "' + respParamsStr[index2_CycLutLen_field] + '", ' + respParamsStr[index2_Anov_field] + ', "' + respParamsStr[index2_AllOvMan_field] + '", "';
                    
                    strSql_fields += 'all_ov_auto, cyc_ov_cd, all_ov_type, ';
                    strSql_values += respParamsStr[index2_AllOvAuto_field] + '", "' + respParamsStr[index2_CycOvCd_field] + '", "' + respParamsStr[index2_AllOvType_field] + '", "';
                    
                    strSql_fields += 'bbt_ov, bbt_cv_ov, bbt_ave_ov, bbt_smc_ov, ';
                    strSql_values += respParamsStr[index2_BbtOv_field] + '", "' + respParamsStr[index2_BbtCvOv_field] + '", "' + respParamsStr[index2_BbtAveOv_field] + '", "' + respParamsStr[index2_BbtSmcOv_field] + '", "';
                    
                    strSql_fields += 'bbt_orig_ov, bbt_36_ov, bbt_meth, lh_ov, ';
                    strSql_values += respParamsStr[index2_BbtOrigOv_field] + '", "' + respParamsStr[index2_Bbt36Ov_field] + '", ' + respParamsStr[index2_BbtMeth_field] + ', "' + respParamsStr[index2_LhOv_field] + '", "';
                    
                    strSql_fields += 'cm_ov, cm_trad_ov, cm_alt_ov, cm_meth, ';
                    strSql_values += respParamsStr[index2_CmOv_field] + '", "' + respParamsStr[index2_CmTradOv_field] + '", "' + respParamsStr[index2_CmAltOv_field] + '", ' + respParamsStr[index2_CmMeth_field] + ', ';
                    
                    if (BUILD_TARGET == BUILD_4_CCL)
                    {
                        strSql_fields += 'cweight, cyc_notes, fwb_st, fwe_st, ';
                        strSql_values += respParamsStr[index2_CycWeight_field] + ', "' + ReplaceQuotes(respParamsStr[index2_CycNotes_field],1) + '", ' + respParamsStr[index2_FwbSt_field] + ', ' + respParamsStr[index2_FweSt_field] + ', ';

                        strSql_fields += 'fwb_in, fwe_in, fwb_man_in, fwe_man_in, ';
                        strSql_values += respParamsStr[index2_FwbIn_field] + ', ' + ReplaceQuotes(respParamsStr[index2_FweIn_field],1) + ', ' + respParamsStr[index2_FwbManIn_field] + ', ' + respParamsStr[index2_FweManIn_field] + ', ';

                        strSql_fields += 'fwb_man_st, fwe_man_st, fwb_vals, fwe_vals, ';
                        strSql_values += respParamsStr[index2_FwbManSt_field] + ', ' + ReplaceQuotes(respParamsStr[index2_FweManSt_field],1) + ', "' + respParamsStr[index2_FwbVals_field] + '", "' + respParamsStr[index2_FweVals_field] + '", ';

                        strSql_fields += 'pc_short, pc_long, pc_num_cyc, pc_early_trise, ';
                        strSql_values += respParamsStr[index2_PcShort_field] + ', ' + ReplaceQuotes(respParamsStr[index2_PcLong_field],1) + ', ' + respParamsStr[index2_PcNumCyc_field] + ', ' + respParamsStr[index2_PcEarlyTrise_field] + ', ';

                        strSql_fields += 'pc_num_tcyc, pc_l_mucpat, inst_notes, id_txt, ';
                        strSql_values += respParamsStr[index2_PcNumTcyc_field] + ', ' + respParamsStr[index2_PCLongMucPatches_field] + ', "' + ReplaceQuotes(respParamsStr[index2_InstNotes_field],1) + '", ' + ReplaceQuotes(respParamsStr[index2_IdTxt_field],1) + '", ';
                    }
                    
                    strSql_fields += 'mon_ov, rhy_ov, saliva_ov, certain, quality, complete, ';
                    strSql_values += '"' + respParamsStr[index2_MonOv_field] + '", "' + respParamsStr[index2_RhyOv_field] + '", "' + respParamsStr[index2_SalivaOv_field] + '", ' + respParamsStr[index2_Certain_field] + ', ' + respParamsStr[index2_Quality_field] + ', ' + respParamsStr[index2_Complete_field] + ', "';
                    
                    strSql_fields += 'fwin_beg, fwin_end, fwin_beg_final, fwin_end_final, fwin_why, mkr) ';
                    strSql_values += respParamsStr[index2_FWinBeg_field] + '", "' + respParamsStr[index2_FWinEnd_field] + '", ' + respParamsStr[index2_FWinBegFinal_field] + ', ' + respParamsStr[index2_FWinEndFinal_field] + ', "' + ReplaceQuotes(respParamsStr[index2_FWinWhy_field], true) + '", 0);';
 
                    QueryDetails.strSql = strSql_fields + strSql_values;
                    QueryDetails.execute_strSql = 1;
                }
            }
            // Execute query
            if (QueryDetails.execute_strSql != 0)
            {
                // Note: suspend_scrolling(true, '') already called above in a blanket call for States 7 thru 14
                tx.executeSql(QueryDetails.strSql, [], successCB, errorCB);
            }
            
            //////////////////////////////////////////////
            // Part 4 of parallel execution is finished //
            //////////////////////////////////////////////
            LoadCalScreen_CyclesDB = 1;
        
            // Check if all parallel parts are complete.  If so, complete by executing the callback
            if ((LoadCalScreen_InterpDB == 1) && (LoadCalScreen_DataRecsDB == 1) && (LoadCalScreen_EventSettingsDB == 1) && (LoadCalScreen_CyclesDB == 1) && (LoadCalScreen_UserInfoDB == 1) && (LoadCalScreen_PregDB == 1))
            {
                // Indicate that this will take ownership of executing the callback.  Part 1 has highest priority, followed by Part 2 and part 3 respectively
                LoadCalScreen_CyclesDB = 0;
                
                // Delay for a moment and check again that a higher priority parallel path didn't already declare ownership of doing the callback
                for (var x=0; x<17; x++)
                {
                    var i=x+2;
                }
                
                // Double check that another parallel task (of higher priority) didn't take ownership by clearing its flag.  We don't want to have more than
                // one parallel thread executing the callback.
                if ((LoadCalScreen_InterpDB == 1) && (LoadCalScreen_DataRecsDB == 1) && (LoadCalScreen_EventSettingsDB == 1))  
                {
                    // Timestamp callback
                    var currTime = new Date();
                    printDebug("State #" + state + " of data_load_fsm parallelism done " + appendTimeStamp(currTime));
                    
                    // Continue execution after parallel execution paths
                    data_load_fsm(LCS_AFTER_PARALLEL_EXEC_STATE_14, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, '', '');
                }              
            }
            break; 
        
        case LCS_PART_5_STATE_12:
LogCall('ac');
            /////////////////////////////////////////////
            // PART 5 - Compare with User_Info_DB //
            /////////////////////////////////////////////
        
            // Analyze server response and update local database with necessary changes 
            var desiredStrStart=new Array();        // Start delimeter to look for in server response
            var desiredStrEnd=new Array();          // End delimeter to look for in server response
            var dbFieldObj=new Array();             // Pointers to objects describing database fields
            var respParamsStr=new Array();          // Placeholder for the parsed response found in the server returned content.
            
            var index2_CycAvg_field;
            var index2_CycSd_field;
            var index2_LutAvg_field;
            var index2_LutSd_field;
            var index2_CycShort_field;
            var index2_CycLong_field;
            var index2_LutShort_field;
            var index2_LutLong_field;
            var index2_EarlyOv_field;
            var index2_LateOv_field;
            var index2_EarlyAnyOv_field;
            var index2_EarlyCmF_field;
            var index2_EarlyBbtOv_field;
            var index2_EarlyMonP_field;
            var index2_LateMonP_field;
            var index2_Height_field;
            
            var newContent = param_3;
            var local_db_counter = 0;
            var len = results.rows.length;
            var rec_date_int;
            
            // Check that the local database query worked.  If it doesn't, display an error and exit
            if (len == 0)
            {
                printInfoMessage('Error re-querying local EventSettings database.  Please try again.  If this problem persists, you may want to try going to the settings page and re-associating with your account.');
                break;
            }
           
            // Prepare variables to extract from response 
            index2_CycAvg_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='CYC_AVG=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbUserInfoDBField_CycAvg;

            index2_LutAvg_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='LUT_AVG=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbUserInfoDBField_LutAvg;

            index2_CycSd_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='CYC_SD=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbUserInfoDBField_CycSd;
            
            index2_LutSd_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='LUT_SD=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbUserInfoDBField_LutSd;
            
            index2_CycShort_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='CYC_SHORT=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbUserInfoDBField_CycShort;
            
            index2_CycLong_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='CYC_LONG=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbUserInfoDBField_CycLong;
            
            index2_LutShort_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='LUT_SHORT=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbUserInfoDBField_LutShort;
            
            index2_LutLong_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='LUT_LONG=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbUserInfoDBField_LutLong;
            
            index2_EarlyOv_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='EARLY_OV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbUserInfoDBField_EarlyOv;
            
            index2_LateOv_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='LATE_OV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbUserInfoDBField_LateOv;
            
            index2_EarlyCmF_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='EARLY_CM_F=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbUserInfoDBField_EarlyCmF;

            index2_EarlyAnyOv_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='EARLY_ANY_OV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbUserInfoDBField_EarlyAnyOv;
            
            index2_EarlyBbtOv_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='EARLY_BBT_OV=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbUserInfoDBField_EarlyBbtOv;
            
            index2_EarlyMonP_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='EARLY_MON_P=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbUserInfoDBField_EarlyMonP;

            index2_LateMonP_field = desiredStrStart.length;
            desiredStrStart[desiredStrStart.length]='LATE_MON_P=';
            desiredStrEnd[desiredStrEnd.length]=" ";
            dbFieldObj[dbFieldObj.length]=dbUserInfoDBField_LateMonP;
            
            if (BUILD_TARGET == BUILD_4_CCL)
            {
                index2_Height_field = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='UHT=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbUserInfoDBField_Height; 
            }            

            // Parse the response to find the desired values
            respParamsStr = parse_server_response(newContent, desiredStrStart, desiredStrEnd);

            // There is one and only 1 record in the local database, so we don't need to go looking for a match like with other databases.  
            local_db_match = 0;
            var QueryDetails = {execute_strSql: 0, strSql: ''};

            // Update the existing database entry and consider "_chg" flag values
            QueryDetails.strSql = 'UPDATE User_Info_DB SET ';
            
            // Create a list of database elements we need to consider
            var given_index;
            var last_read_dirty_prefix = '';
            var last_read_chg = '';
            var last_read_pnd = '';
            var relevantIndices=new Array();  

            relevantIndices[relevantIndices.length] = index2_CycAvg_field;
            relevantIndices[relevantIndices.length] = index2_CycSd_field;
            relevantIndices[relevantIndices.length] = index2_LutAvg_field;
            relevantIndices[relevantIndices.length] = index2_LutSd_field;
            relevantIndices[relevantIndices.length] = index2_CycShort_field;
            relevantIndices[relevantIndices.length] = index2_CycLong_field;
            relevantIndices[relevantIndices.length] = index2_LutShort_field;
            relevantIndices[relevantIndices.length] = index2_LutLong_field;
            relevantIndices[relevantIndices.length] = index2_EarlyOv_field;
            relevantIndices[relevantIndices.length] = index2_LateOv_field;
            relevantIndices[relevantIndices.length] = index2_EarlyAnyOv_field;
            relevantIndices[relevantIndices.length] = index2_EarlyCmF_field;
            relevantIndices[relevantIndices.length] = index2_EarlyBbtOv_field;
            relevantIndices[relevantIndices.length] = index2_EarlyMonP_field;
            relevantIndices[relevantIndices.length] = index2_LateMonP_field;
            if (BUILD_TARGET == BUILD_4_CCL)
            {
                relevantIndices[relevantIndices.length] = index2_Height_field;
            }
            
            // Call a function to walk through the relevantIndices and build the appropriate query string and indicate if it should execute using QueryDetails
            QueryDetails = loopThruRelevantIndicesAndStoreLocally(relevantIndices, respParamsStr, dbFieldObj, results, local_db_counter, QueryDetails);
            QueryDetails.strSql += ' WHERE id=' + results.rows.item(local_db_counter)['id'] + ';';

            // Execute query
            if (QueryDetails.execute_strSql != 0)
            {
                // Note: suspend_scrolling(true, '') already called above in a blanket call for States 7 thru 14
                tx.executeSql(QueryDetails.strSql, [], successCB, errorCB);
            }
            
            //////////////////////////////////////////////
            // Part 5 of parallel execution is finished //
            //////////////////////////////////////////////
            LoadCalScreen_UserInfoDB = 1;
            var currTime = new Date(); 
            g_LoadTimestamps['events'] = currTime.getTime();        // Timestamp events so there's no need to reload them again too soon

            // Check if all parallel parts are complete.  If so, complete by executing the callback
            if ((LoadCalScreen_InterpDB == 1) && (LoadCalScreen_DataRecsDB == 1) && (LoadCalScreen_EventSettingsDB == 1) && (LoadCalScreen_CyclesDB == 1) && (LoadCalScreen_UserInfoDB == 1) && (LoadCalScreen_PregDB == 1))
            {
                // Indicate that this will take ownership of executing the callback.  Part 1 has highest priority, followed by Part 2 and part 3 respectively
                LoadCalScreen_UserInfoDB = 0;
                
                // Delay for a moment and check again that a higher priority parallel path didn't already declare ownership of doing the callback
                for (var x=0; x<28; x++)
                {
                    var i=x+2;
                }
                
                // Double check that another parallel task (of higher priority) didn't take ownership by clearing its flag.  We don't want to have more than
                // one parallel thread executing the callback.
                if ((LoadCalScreen_InterpDB == 1) && (LoadCalScreen_DataRecsDB == 1) && (LoadCalScreen_EventSettingsDB == 1) && (LoadCalScreen_CyclesDB == 1))  
                {
                    // Timestamp callback
                    var currTime = new Date();
                    printDebug("State #" + state + " of data_load_fsm parallelism done " + appendTimeStamp(currTime));
                
                    // Continue execution after parallel execution paths
                    data_load_fsm(LCS_AFTER_PARALLEL_EXEC_STATE_14, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, '', '');
                }              
            }
            break;  

        case LCS_PART_6_STATE_13:
LogCall('ad');
            /////////////////////////////////////////////
            // PART 6 - Compare with Preg_DB //
            /////////////////////////////////////////////
        
            // Analyze server response and update local database with necessary changes 
            var desiredStrStart=new Array();        // Start delimeter to look for in server response
            var desiredStrEnd=new Array();          // End delimeter to look for in server response
            var dbFieldObj=new Array();             // Pointers to objects describing database fields
            var respParamsStr=new Array();          // Placeholder for the parsed response found in the server returned content.

            var index2_PregStart_field=new Array();
            var index2_PregLen_field=new Array();
            var index2_PregStatus_field=new Array();

            var newContent = param_3;
            var local_db_counter = 0;
            var len = results.rows.length;
            var local_preg_date_int = -1;
            if (len > local_db_counter)
            {
                local_preg_date_int = results.rows.item(local_db_counter)['preg_started'];
            }
            var rem_preg_date_int = -1;
            
            var deleteList=new Array();
            var deleteIndex = 0;

            // Prepare variables to extract from response
            desiredStrStart[desiredStrStart.length]='TOTAL_PREG=';
            desiredStrEnd[desiredStrEnd.length]=" ";

            // Initially parse the response to find the number of pregs
            respParamsStr = parse_server_response(newContent, desiredStrStart, desiredStrEnd);
            var num_pregs = respParamsStr[0];
            desiredStrStart.length = 0;
            desiredStrEnd.length = 0;
            for (var i=1; i<=num_pregs; i++)
            {
                index2_PregStart_field[i-1] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='PREG_' + i + '_STARTED=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbPregDBField_PregStart;

                index2_PregLen_field[i-1] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='PREG_' + i + '_LEN=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbPregDBField_PregLen;

                index2_PregStatus_field[i-1] = desiredStrStart.length;
                desiredStrStart[desiredStrStart.length]='PREG_' + i + '_STATUS=';
                desiredStrEnd[desiredStrEnd.length]=" ";
                dbFieldObj[dbFieldObj.length]=dbPregDBField_PregStatus;
            }
            
            // Parse the response to find the desired values
            if (num_pregs > 0)
            {
                respParamsStr = parse_server_response(newContent, desiredStrStart, desiredStrEnd);
            }
  
            var QueryDetails = {execute_strSql: 0, strSql: ''};            
            // Compare to database        
            for (var i=num_pregs; i>=1; i--)
            {
                local_db_match = 0;
                rem_preg_date_int = DateToNumber(ConvertYYYY_MM_DD_toDate(respParamsStr[index2_PregStart_field[i-1]]));
                respParamsStr[index2_PregStart_field[i-1]] = rem_preg_date_int;
                // See if the local database already has a value that matches this
                if (local_db_counter < len)
                {
                    local_preg_date_int = results.rows.item(local_db_counter)['preg_started'];
                    // Skip over entries that should be deleted
                    while ((local_db_counter < len) && (local_preg_date_int < rem_preg_date_int))
                    {
                        deleteList[deleteIndex++] = local_preg_date_int;
                        local_db_counter += 1;
                        if (local_db_counter < len)
                        {
                            local_preg_date_int = results.rows.item(local_db_counter)['preg_started'];
                        }
                    }
                    
                    if (local_preg_date_int == rem_preg_date_int)
                    {
                        // This is a match.  We must update the existing database entry and consider "_chg" flag values
                        QueryDetails.strSql = 'UPDATE Preg_DB SET ';
                        
                        // Create a list of database elements we need to consider
                        var given_index;
                        var last_read_dirty_prefix = '';
                        var last_read_chg = '';
                        var last_read_pnd = '';
                        var relevantIndices=new Array();                     
                        
                        relevantIndices[relevantIndices.length] = index2_PregStart_field[i-1];
                        relevantIndices[relevantIndices.length] = index2_PregLen_field[i-1];
                        relevantIndices[relevantIndices.length] = index2_PregStatus_field[i-1];
                        
                        // Call a function to walk through the relevantIndices and build the appropriate query string and indicate if it should execute using QueryDetails
                        QueryDetails = loopThruRelevantIndicesAndStoreLocally(relevantIndices, respParamsStr, dbFieldObj, results, local_db_counter, QueryDetails);
                        QueryDetails.strSql += ' WHERE preg_started=' + local_preg_date_int + ';';
                        
                        local_db_match = 1; 
                        local_db_counter += 1;
                    }
                }
                // Just create a new database entry if one didn't already exist
                if (local_db_match == 0)
                {
                    // Make sure the server returned something by checking a couple of parameters we will put in the local database
                    if ((respParamsStr[index2_PregLen_field] != '') && (respParamsStr[index2_PregStatus_field] != ''))
                    {
                        var strSql_fields;
                        var strSql_values;
                        
                        // No match.  We must create a database entry. 
                        strSql_fields = 'INSERT INTO Preg_DB (preg_started, ';
                        strSql_values = 'VALUES (' + rem_preg_date_int + ', ';
                        
                        strSql_fields += 'pdata_chg, pdata_pnd, ';
                        strSql_values += '0, 0, ';

                        strSql_fields += 'preg_len, preg_status) ';
                        strSql_values += respParamsStr[index2_PregLen_field[i-1]] + ', "' + respParamsStr[index2_PregStatus_field[i-1]] + '");';
     
                        QueryDetails.strSql = strSql_fields + strSql_values;
                        QueryDetails.execute_strSql = 1;
                    }
                }
                // Execute query
                if (QueryDetails.execute_strSql != 0)
                {
                    // Note: suspend_scrolling(true, '') already called above in a blanket call for States 7 thru 14
                    tx.executeSql(QueryDetails.strSql, [], successCB, errorCB);
                    QueryDetails.execute_strSql = 0;
                }
                
            }
            // Left overs need deleted
            while ((local_db_counter < len) && (local_preg_date_int > rem_preg_date_int))
            {
                deleteList[deleteIndex++] = local_preg_date_int;
                local_db_counter += 1;
                if (local_db_counter < len)
                {
                    local_preg_date_int = results.rows.item(local_db_counter)['preg_started'];
                }
            }
            
            // Delete unmatched local entries
            QueryDetails.strSql = '';
            for (var i=0; i<deleteIndex; i++)
            {
                // No match.  We must delete database entry (called separate to avoid error). 
                QueryDetails.strSql = 'DELETE FROM Preg_DB WHERE preg_started=' + deleteList[i] + '; ';
                tx.executeSql(QueryDetails.strSql, [], successCB, errorCB);
            }
            
            //////////////////////////////////////////////
            // Part 6 of parallel execution is finished //
            //////////////////////////////////////////////
            LoadCalScreen_PregDB = 1;
            var currTime = new Date(); 
            g_LoadTimestamps['events'] = currTime.getTime();        // Timestamp events so there's no need to reload them again too soon

            // Check if all parallel parts are complete.  If so, complete by executing the callback
            if ((LoadCalScreen_InterpDB == 1) && (LoadCalScreen_DataRecsDB == 1) && (LoadCalScreen_EventSettingsDB == 1) && (LoadCalScreen_CyclesDB == 1) && (LoadCalScreen_UserInfoDB == 1) && (LoadCalScreen_PregDB == 1))
            {
                // Indicate that this will take ownership of executing the callback.  Part 1 has highest priority, followed by Part 2 and part 3 respectively
                LoadCalScreen_PregDB = 0;
                
                // Delay for a moment and check again that a higher priority parallel path didn't already declare ownership of doing the callback
                for (var x=0; x<28; x++)
                {
                    var i=x+2;
                }
                
                // Double check that another parallel task (of higher priority) didn't take ownership by clearing its flag.  We don't want to have more than
                // one parallel thread executing the callback.
                if ((LoadCalScreen_InterpDB == 1) && (LoadCalScreen_DataRecsDB == 1) && (LoadCalScreen_EventSettingsDB == 1) && (LoadCalScreen_CyclesDB == 1) && (LoadCalScreen_UserInfoDB == 1))  
                {
                    // Timestamp callback
                    var currTime = new Date();
                    printDebug("State #" + state + " of data_load_fsm parallelism done " + appendTimeStamp(currTime));
                
                    // Continue execution after parallel execution paths
                    data_load_fsm(LCS_AFTER_PARALLEL_EXEC_STATE_14, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, '', '');
                }              
            }
            break;  

        case LCS_AFTER_PARALLEL_EXEC_STATE_14:
LogCall('ae');
            printStatus("Rendering");
            
            // Timestamp when we loaded this specific mm-yyyy calendar so we don't reload it again for no reason
            var currTime = new Date(); 
            g_LoadTimestamps[calendar_loaded_date.getMonth() + '-' + calendar_loaded_date.getFullYear()] = currTime.getTime();
        
            // The following database query is just to create delay to to ensure there is adequate 
            // time between the updates being processed (STEP 3) and the new screen being rendered (data loaded in STEP 4)
            // This could be deemed unnecessary later.
            strSql = 'SELECT * FROM Data_Recs_DB WHERE 1 LIMIT 1;';
            // Note: suspend_scrolling(true, '') already called above in a blanket call for States 7 thru 14
            tx.executeSql(  strSql, 
                            [], 
                            function(tx, results) 
                            {
                                data_load_fsm(state+1, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, results, '');
                            },
                            function(tx, err)
                            {
                                // Since this step is unnecessary, we want to proceed even if there is an error because nobody needs the result
                                data_load_fsm(state+1, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, '', '');
                                errorCB(tx, err);
                            }
                         );
            break;
            
        //////////////////////////////////////////////////////
        // STEP 4 - Re-Initialize Screen Given Data Updates //
        //////////////////////////////////////////////////////
        case 15:
LogCall('af'); 
            // Turn back on scrolling (it as off for States 7-14 of the data_load_fsm)
            suspend_scrolling(false, 0);

            // Re-render user interface again now that local database has been refreshed
            if (targetCalendar_OR_Analysis_NOT)
            {
                // This call is for the calendar level of screens.  Load content for those.
                RenderCalendarFromLocalDatabase(     1,
                                        function(tx) 
                                        {
                                            data_load_fsm(state+1, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, '', '', false);
                                        },
                                        tx,
                                        '', 
                                        true
                                   );
            }
            else
            {
                // This call is for the Charting & Analysis level of screens.  Load content for those.
                RenderAnalysisScreensFromLocalDatabase(     1,
                                        function(tx) 
                                        {
                                            data_load_fsm(state+1, targetCalendar_OR_Analysis_NOT, start_date_as_int, end_date_as_int, caller_url, tx, '', '', false);
                                        },
                                        tx,
                                        '',
                                        '',
                                        true,
                                        start_date_as_int + global_selected_cycle_offset, 
                                        end_date_as_int,
                                        '',
                                        ''
                                   );
            }
            break;
            
        case 16:
LogCall('ag');
            printStatus('');
            printDebug('ALL DONE!');
            break;
        
        default:
            // Should not happen
            printDebug('Error - Callback State Machine calling default (state=' + state + ')');
            printInfoMessage('Invalid FSM state in LoadCal: ' + state);
        }
        return false;
    }
    // Function prints status messages if the app is offline and therefore has limited functionality.  Returns "true"
    // if this is the case and "false" if it is safe to try to connect to the remote server.
    function CheckAndWarnIfOfflineOnly()
    {
        if (global_EnableRemoteAnalysis == 0)
        {
            printOfflineStatus();
            printInfoMessage('The app has been configured to disable remote analysis.  Functionality is very limited with this off.  It is recommended that you change your settings and turn Remote Analysis back on.  Without it, newly recorded data cannot be analyzed and fertility is not being interpreted.'); 
            return true;
        }
        else if (global_online_status == 0)
        {
            printOfflineStatus();
            printInfoMessage('Without internet access, the app cannot update the analysis.  The last available analysis will be shown.  Newly recorded data cannot be analyzed and fertility is therefore not being interpreted.');
            return true;
        }
        else
        {
            // Clear messages
            clearInfoMessage();
            return false;
        }
    }
    function WarnIfNoActiveSubscription()
    {
        printInfoMessage('Some features are disabled because of your subscription status (e.g. fertility shading, advanced fertility and analysis, etc).  The remote server requires a Premium Subscription and therefore cannot be displayed.  For more information, click on the Settings button and swipe to the subscription screen.'); 
    }
    function IfEmptyAssignAlternate(value_returned, neutral_value)
    {
        if (value_returned == '')
        {
            return neutral_value;
        }
        return value_returned;
    }
    
